from mteb.models.model_meta import ModelMeta, ScoringFunction
from mteb.models.sentence_transformer_wrapper import SentenceTransformerEncoderWrapper

b1ade_training_data = {
    # We are in the process of submitting a paper outlining our process of creating b1ade using model merging and knowledge distillation.
    # Similar to mixedbread models, we do not train on any data (except the MSMarco training split) of MTEB.
    "MSMARCO",
}

b1ade_embed = ModelMeta(
    loader=SentenceTransformerEncoderWrapper,
    name="w601sxs/b1ade-embed",
    model_type=["dense"],
    languages=["eng-Latn"],
    revision="3bdac13927fdc888b903db93b2ffdbd90b295a69",
    open_weights=True,
    release_date="2025-03-10",
    n_parameters=335141888,
    n_embedding_parameters=31_254_528,
    memory_usage_mb=1278,
    embed_dim=1024,
    license="mit",
    max_tokens=4096,
    reference="https://huggingface.co/w601sxs/b1ade-embed",
    similarity_fn_name=ScoringFunction.COSINE,
    framework=["Sentence Transformers", "PyTorch", "Transformers", "safetensors"],
    use_instructions=False,
    public_training_code=None,
    public_training_data=None,
    training_datasets=b1ade_training_data,
    adapted_from="BAAI/bge-large-en-v1.5",
    # Also (based on model merging)
    # bert-large-uncased
    # WhereIsAI/UAE-Large-V1
    # BAAI/bge-large-en-v1.5
    # mixedbread-ai/mxbai-embed-large-v1
    # avsolatorio/GIST-large-Embedding-v0
    citation="""
    @misc{bigscience_workshop_2022,
    author       = { {Shreyas Subramanian} },
    title        = { {b1ade series of models} },
    year         = 2024,
    url          = { https://huggingface.co/w601sxs/b1ade-embed },
    publisher    = { Hugging Face }
}
""",
)



from mteb.models.instruct_wrapper import InstructSentenceTransformerModel
from mteb.models.model_meta import ModelMeta
from mteb.models.sentence_transformer_wrapper import SentenceTransformerEncoderWrapper
from mteb.types import PromptType

SARASHINA_V2_INSTRUCTIONS = {
    "Retrieval": {
        "query": "クエリを与えるので、もっともクエリに意味が似ている一節を探してください。",
        "document": "text: ",
    },
    "Reranking": {
        "query": "クエリを与えるので、もっともクエリに意味が似ている一節を探してください。",
        "document": "text: ",
    },
    "Classification": "与えられたドキュメントを適切なカテゴリに分類してください。",
    "Clustering": "与えられたドキュメントのトピックまたはテーマを特定してください。",
    # optimization regarding JMTEB
    "LivedoorNewsClustering.v2": "与えられたニュース記事のトピックを特定してください。",
    "MewsC16JaClustering": "与えられたニュース記事のトピックを特定してください。",
    "SIB200ClusteringS2S": "与えられたテキストのトピックを特定してください。",
    "AmazonReviewsClassification": "与えられたAmazonレビューを適切な評価カテゴリに分類してください。",
    "AmazonCounterfactualClassification": "与えられたAmazonのカスタマーレビューのテキストを反事実か反事実でないかに分類してください。",
    "MassiveIntentClassification": "ユーザーの発話をクエリとして与えるので、ユーザーの意図を見つけてください。",
    "MassiveScenarioClassification": "ユーザーの発話をクエリとして与えるので、ユーザーシナリオを見つけてください。",
    "JapaneseSentimentClassification": "与えられたテキストの感情極性をポジティブ(1)かネガティブか(0)に分類してください。",
    "SIB200Classification": "与えられたテキストのトピックを特定してください。",
    "WRIMEClassification": "与えられたテキストの感情極性（-2:強いネガティブ、-1:ネガティブ、0:ニュートラル、1:ポジティブ、2:強いポジティブ）を分類してください。",
    "JSTS": "クエリを与えるので，もっともクエリに意味が似ている一節を探してください。",
    "JSICK": "クエリを与えるので，もっともクエリに意味が似ている一節を探してください。",
    "JaqketRetrieval": {
        "query": "質問を与えるので、その質問に答えるのに役立つWikipediaの文章を検索してください。",
        "document": "text: ",
    },
    "MrTidyRetrieval": {
        "query": "質問を与えるので、その質問に答えるWikipediaの文章を検索するしてください。",
        "document": "text: ",
    },
    "JaGovFaqsRetrieval": {
        "query": "質問を与えるので、その質問に答えるのに役立つ関連文書を検索してください。",
        "document": "text: ",
    },
    "NLPJournalTitleAbsRetrieval.V2": {
        "query": "論文のタイトルを与えるので、タイトルに対応する要約を検索してください。",
        "document": "text: ",
    },
    "NLPJournalTitleIntroRetrieval.V2": {
        "query": "論文のタイトルを与えるので、タイトルに対応する要約を検索してください。",
        "document": "text: ",
    },
    "NLPJournalAbsIntroRetrieval.V2": {
        "query": "論文の序論を与えるので、序論に対応する全文を検索してください。",
        "document": "text: ",
    },
    "NLPJournalAbsArticleRetrieval.V2": {
        "query": "論文の序論を与えるので、序論に対応する全文を検索してください。",
        "document": "text: ",
    },
    "JaCWIRRetrieval": {
        "query": "記事のタイトルを与えるので、そのタイトルと合っている記事の中身を検索してください。",
        "document": "text: ",
    },
    "MIRACLRetrieval": {
        "query": "質問を与えるので、その質問に答えるのに役立つ関連文書を検索してください。",
        "document": "text: ",
    },
    "MintakaRetrieval": {
        "query": "質問を与えるので、その質問に答えられるテキストを検索してください。",
        "document": "text: ",
    },
    "MultiLongDocRetrieval": {
        "query": "質問を与えるので、その質問に答えるのに役立つWikipediaの文章を検索してください。",
        "document": "text: ",
    },
    "ESCIReranking": {
        "query": "クエリを与えるので、与えられたWeb検索クエリに答える関連文章を検索してください。",
        "document": "text: ",
    },
    "JQaRAReranking": {
        "query": "質問を与えるので、その質問に答えるのに役立つWikipediaの文章を検索してください。",
        "document": "text: ",
    },
    "JaCWIRReranking": {
        "query": "記事のタイトルを与えるので、そのタイトルと合っている記事の中身を検索してください。",
        "document": "text: ",
    },
    "MIRACLReranking": {
        "query": "質問を与えるので、その質問に答えるのに役立つ関連文書を検索してください。",
        "document": "text: ",
    },
    "MultiLongDocReranking": {
        "query": "質問を与えるので、その質問に答えるのに役立つWikipediaの文章を検索してください。",
        "document": "text: ",
    },
}


def sarashina_instruction_template(
    instruction: str, prompt_type: PromptType | None = None
) -> str:
    """Instruction template for Sarashina v2 model.

    Returns the instruction as-is since the prompts already contain the full format.
    For document prompts, returns the instruction directly (e.g., "text: ").
    """
    if not instruction:
        return ""
    if prompt_type == PromptType.document:
        return "text: "
    return f"task: {instruction}\nquery: "


sbintuitions_sarashina_embedding_v2_1b = ModelMeta(
    loader=InstructSentenceTransformerModel,
    loader_kwargs=dict(
        instruction_template=sarashina_instruction_template,
        apply_instruction_to_passages=True,
        prompts_dict=SARASHINA_V2_INSTRUCTIONS,
        max_seq_length=8192,
    ),
    name="sbintuitions/sarashina-embedding-v2-1b",
    model_type=["dense"],
    languages=["jpn-Jpan"],
    open_weights=True,
    revision="1f3408afaa7b617e3445d891310a9c26dd0c68a5",
    release_date="2025-07-30",
    n_parameters=1_224_038_144,
    n_embedding_parameters=183_500_800,
    memory_usage_mb=4669,
    embed_dim=1792,
    license="https://huggingface.co/sbintuitions/sarashina-embedding-v2-1b/blob/main/LICENSE",
    max_tokens=8192,
    reference="https://huggingface.co/sbintuitions/sarashina-embedding-v2-1b",
    similarity_fn_name="cosine",
    framework=["Sentence Transformers", "PyTorch", "safetensors", "Transformers"],
    use_instructions=True,
    adapted_from="sbintuitions/sarashina2.2-1b",
    superseded_by=None,
    training_datasets={"NQ", "MrTidyRetrieval"},
    public_training_code=None,
    public_training_data="https://huggingface.co/sbintuitions/sarashina-embedding-v1-1b",
    citation=None,
    contacts=["Sraym1217", "akiFQC", "lsz05"],
)

sbintuitions_sarashina_embedding_v1_1b = ModelMeta(
    loader=SentenceTransformerEncoderWrapper,
    name="sbintuitions/sarashina-embedding-v1-1b",
    model_type=["dense"],
    languages=["jpn-Jpan"],
    open_weights=True,
    revision="d060fcd8984075071e7fad81baff035cbb3b6c7e",
    release_date="2024-11-22",
    n_parameters=1_224_038_144,
    n_embedding_parameters=183_500_800,
    memory_usage_mb=4669,
    embed_dim=1792,
    license="https://huggingface.co/sbintuitions/sarashina-embedding-v1-1b/blob/main/LICENSE",
    max_tokens=8192,
    reference="https://huggingface.co/sbintuitions/sarashina-embedding-v1-1b",
    similarity_fn_name="cosine",
    framework=["Sentence Transformers", "PyTorch", "safetensors", "Transformers"],
    use_instructions=False,
    adapted_from="sbintuitions/sarashina2.1-1b",
    superseded_by="sbintuitions/sarashina-embedding-v2-1b",
    training_datasets={"NQ", "MrTidyRetrieval"},
    public_training_code=None,
    public_training_data="https://huggingface.co/sbintuitions/sarashina-embedding-v1-1b",
    citation=None,
    contacts=["akiFQC", "lsz05"],
)



from .spanish_news_classification import (
    SpanishNewsClassification,
    SpanishNewsClassificationV2,
)
from .spanish_sentiment_classification import (
    SpanishSentimentClassification,
    SpanishSentimentClassificationV2,
)

__all__ = [
    "SpanishNewsClassification",
    "SpanishNewsClassificationV2",
    "SpanishSentimentClassification",
    "SpanishSentimentClassificationV2",
]



from __future__ import annotations

import logging
import warnings
from typing import TYPE_CHECKING, Any

import numpy as np
import torch
from tqdm.auto import tqdm

from mteb._requires_package import requires_audio_dependencies, requires_package
from mteb.models.abs_encoder import AbsEncoder
from mteb.models.model_meta import ModelMeta

if TYPE_CHECKING:
    from torch.utils.data import DataLoader

    from mteb import TaskMetadata
    from mteb.types import Array, BatchedInput, PromptType
    from mteb.types._encoder_io import AudioInput

logger = logging.getLogger(__name__)


def vggish_loader(*args, **kwargs):
    """Factory function to create a VGGish model wrapper."""
    requires_package(
        vggish_loader,
        "torch_vggish_yamnet",
        "google/vggish",
        "pip install 'mteb[torch-vggish-yamnet]'",
    )
    import torchaudio
    from torch_vggish_yamnet import vggish
    from torch_vggish_yamnet.input_proc import WaveformToInput

    class VGGishWrapper(AbsEncoder):
        def __init__(
            self,
            device: str = "cuda" if torch.cuda.is_available() else "cpu",
            max_audio_length_seconds: float = 30.0,
            **kwargs: Any,
        ):
            requires_audio_dependencies()
            self.device = device
            self.max_audio_length_seconds = max_audio_length_seconds
            self.model = vggish.get_vggish(with_classifier=False, pretrained=True)
            self.model.eval().to(self.device)

            self.converter = WaveformToInput()
            self.sampling_rate = 16000
            self.embed_dim = 128
            self.min_samples = int(0.96 * self.sampling_rate)  # 15,360 samples

        def _resample_audio(self, audio, source_rate):
            """Resample audio to target sampling rate."""
            if source_rate != self.sampling_rate:
                resampler = torchaudio.transforms.Resample(
                    source_rate, self.sampling_rate
                )
                return resampler(audio)
            return audio

        def _normalize_audio(self, audio):
            """Normalize and pad audio to minimum length."""
            if isinstance(audio, np.ndarray):
                audio = torch.from_numpy(audio)

            audio = audio.float().squeeze()
            if audio.ndim > 1:
                audio = audio.mean(dim=0)

            # Apply audio truncation
            max_length = int(self.max_audio_length_seconds * self.sampling_rate)
            if audio.shape[-1] > max_length:
                audio = audio[..., :max_length]

            # Normalize to [-1.0, 1.0]
            if audio.numel() > 0 and audio.abs().max() > 1.0:
                audio = audio / audio.abs().max()

            # Pad to minimum length
            if audio.shape[-1] < self.min_samples:
                pad_amount = self.min_samples - audio.shape[-1]
                audio = torch.nn.functional.pad(audio, (0, pad_amount))

            return audio

        def _prepare_input_tensor(self, audio_data):
            """Convert audio to VGGish input format and handle tensor dimensions."""
            if isinstance(audio_data, np.ndarray):
                audio_data = torch.from_numpy(audio_data)

            if audio_data.ndim == 1:
                audio_data = audio_data.unsqueeze(0)

            input_tensor = self.converter(audio_data.float(), self.sampling_rate).to(
                self.device
            )

            # Handle different tensor dimensions
            if input_tensor.dim() == 4 and input_tensor.shape[1] == 3:
                # [batch, 3, height, width] -> [batch, 1, height, width]
                input_tensor = input_tensor.mean(dim=1, keepdim=True)
            elif input_tensor.dim() == 3:
                if input_tensor.shape[0] == 3:
                    # [3, height, width] -> [1, 1, height, width]
                    input_tensor = input_tensor.mean(dim=0, keepdim=True).unsqueeze(0)
                else:
                    # [batch, height, width] -> [batch, 1, height, width]
                    input_tensor = input_tensor.unsqueeze(1)
            elif input_tensor.dim() == 2:
                # [height, width] -> [1, 1, height, width]
                input_tensor = input_tensor.unsqueeze(0).unsqueeze(0)

            return input_tensor

        def get_audio_embeddings(
            self,
            inputs: DataLoader[AudioInput],
            show_progress_bar=True,
            **kwargs,
        ):
            """Generate embeddings for audio inputs."""
            all_embeddings = []

            for batch in tqdm(
                inputs,
                desc="Processing audio",
                disable=not show_progress_bar,
            ):
                batch_embeddings = []

                for a in batch["audio"]:
                    array = torch.tensor(a["array"], dtype=torch.float32)
                    sr = a["sampling_rate"]
                    if sr is None:
                        warnings.warn(
                            f"No sampling_rate provided for an audio sample. "
                            f"Assuming {self.sampling_rate} Hz (model default)."
                        )
                        sr = self.sampling_rate

                    # Resample if needed
                    audio = self._resample_audio(array.float(), sr)
                    # Normalize
                    audio = self._normalize_audio(audio)

                    with torch.no_grad():
                        input_tensor = self._prepare_input_tensor(audio)
                        embedding = self.model(input_tensor)

                        # Use mean pooling if needed
                        if len(embedding.shape) > 1:
                            embedding = torch.mean(embedding, dim=0)

                        batch_embeddings.append(embedding.cpu().detach().numpy())

                all_embeddings.extend(batch_embeddings)

            return np.array(all_embeddings)

        def encode(
            self,
            inputs: DataLoader[BatchedInput],
            *,
            task_metadata: TaskMetadata,
            hf_split: str,
            hf_subset: str,
            prompt_type: PromptType | None = None,
            **kwargs: Any,
        ) -> Array:
            if "audio" not in inputs.dataset.features:
                raise ValueError("VGGishWrapper only supports audio inputs.")
            return self.get_audio_embeddings(inputs, **kwargs)

    return VGGishWrapper(**kwargs)


vggish = ModelMeta(
    loader=vggish_loader,
    name="google/vggish",
    languages=["eng-Latn"],
    open_weights=True,
    revision="1",
    release_date="2019-06-13",
    max_tokens=float("inf"),
    n_parameters=72_141_184,
    n_embedding_parameters=0,
    memory_usage_mb=275,
    embed_dim=128,
    license="apache-2.0",
    reference="https://github.com/tensorflow/models/tree/master/research/audioset/vggish",
    similarity_fn_name="cosine",
    framework=["PyTorch"],
    use_instructions=False,
    public_training_code="https://github.com/tensorflow/models/tree/master/research/audioset/vggish",
    public_training_data="https://research.google.com/audioset/",
    training_datasets={
        "AudioSet",
    },
    modalities=["audio"],
    citation="""
@inproceedings{hershey2017cnn,
    author = {Hershey, Shawn and Chaudhuri, Sourish and Ellis, Daniel P. W. and Gemmeke, Jort F. and Jansen, Aren and Moore, R. Channing and Plakal, Manoj and Platt, Devin and Saurous, Rif A. and Seybold, Bryan and Slaney, Malcolm and Weiss, Ron J. and Wilson, Kevin},
    title = {CNN architectures for large-scale audio classification},
    year = {2017},
    publisher = {IEEE Press},
    url = {https://doi.org/10.1109/ICASSP.2017.7952132},
    doi = {10.1109/ICASSP.2017.7952132},
    booktitle = {2017 IEEE International Conference on Acoustics, Speech and Signal Processing (ICASSP)},
    pages = {131–135},
    numpages = {5},
    location = {New Orleans, LA, USA}
}
""",
)



from __future__ import annotations

import logging
from typing import TYPE_CHECKING, Any

import torch
import torch.nn.functional as F
from tqdm.autonotebook import tqdm

from mteb._requires_package import requires_image_dependencies, requires_package
from mteb.models.abs_encoder import AbsEncoder
from mteb.models.model_meta import ModelMeta, ScoringFunction

if TYPE_CHECKING:
    from torch.utils.data import DataLoader

    from mteb.abstasks.task_metadata import TaskMetadata
    from mteb.types import Array, BatchedInput, PromptType

logger = logging.getLogger(__name__)

NANOVDR_CITATION = """@article{nanovdr2026,
  title={NanoVDR: Distilling a 2B Vision-Language Retriever into a 70M Text-Only Encoder for Visual Document Retrieval},
  author={Liu, Zhuchenyang and Zhang, Yao and Xiao, Yu},
  journal={arXiv preprint arXiv:2603.12824},
  year={2026}
}"""

QUERY_INSTRUCTION = "Find a document image that matches the given query."


class NanoVDRWrapper(AbsEncoder):
    """Asymmetric retrieval wrapper for NanoVDR.

    Routes queries to a lightweight text-only SentenceTransformer student
    and documents to the frozen Qwen3-VL-Embedding-2B VLM teacher.
    """

    def __init__(
        self,
        model_name: str,
        revision: str | None = None,
        device: str | None = None,
        **kwargs: Any,
    ):
        requires_package(
            self,
            "sentence_transformers",
            model_name,
            "pip install sentence-transformers",
        )

        self.device = device or (
            "cuda"
            if torch.cuda.is_available()
            else "mps"
            if torch.backends.mps.is_available()
            else "cpu"
        )

        # Query encoder: lightweight text-only student
        from sentence_transformers import SentenceTransformer

        self.query_model = SentenceTransformer(
            model_name,
            revision=revision,
            device=self.device,
        )

        # Document encoder: frozen VLM teacher (lazy-loaded on first use)
        self._doc_model = None
        self._doc_processor = None

    def _load_teacher(self) -> None:
        """Lazily load the Qwen3-VL-Embedding-2B teacher for document encoding."""
        if self._doc_model is not None:
            return

        requires_image_dependencies()
        requires_package(
            self,
            "transformers",
            "Qwen/Qwen3-VL-Embedding-2B",
            "pip install 'mteb[qwen-vl]'",
        )
        requires_package(
            self,
            "qwen_vl_utils",
            "Qwen/Qwen3-VL-Embedding-2B",
            "pip install 'mteb[qwen-vl]'",
        )

        from transformers.models.qwen3_vl.processing_qwen3_vl import Qwen3VLProcessor

        from mteb.models.model_implementations.qwen3_vl_embedding_models import (
            _build_qwen3_vl_for_embedding_class,
        )

        qwen3_vl_cls = _build_qwen3_vl_for_embedding_class()
        self._doc_model = qwen3_vl_cls.from_pretrained(
            "Qwen/Qwen3-VL-Embedding-2B",
        ).to(self.device)
        self._doc_model.eval()
        self._doc_processor = Qwen3VLProcessor.from_pretrained(
            "Qwen/Qwen3-VL-Embedding-2B",
            padding_side="right",
        )

    def _encode_queries(
        self,
        inputs: DataLoader[BatchedInput],
        show_progress_bar: bool = True,
    ) -> Array:
        all_texts = [text for batch in inputs for text in batch["text"]]
        return self.query_model.encode(
            all_texts,
            show_progress_bar=show_progress_bar,
            convert_to_numpy=False,
        )

    def _encode_documents(
        self,
        inputs: DataLoader[BatchedInput],
        show_progress_bar: bool = True,
    ) -> Array:
        """Encode document page images using the Qwen3-VL teacher."""
        import unicodedata

        from qwen_vl_utils.vision_process import process_vision_info

        self._load_teacher()

        import torchvision.transforms.functional as tv_functional
        from PIL import Image

        instruction = QUERY_INSTRUCTION.strip()
        if instruction and not unicodedata.category(instruction[-1]).startswith("P"):
            instruction = instruction + "."

        all_embeddings: list[torch.Tensor] = []
        with torch.no_grad():
            for batch in tqdm(
                inputs, disable=not show_progress_bar, desc="Encoding docs"
            ):
                contains_image = "image" in batch and batch["image"] is not None
                contains_text = "text" in batch

                batch_size = len(batch["image"] if contains_image else batch["text"])

                conversations = []
                for i in range(batch_size):
                    content: list[dict[str, Any]] = []
                    if contains_image:
                        img = batch["image"][i]
                        if isinstance(img, Image.Image):
                            pil_img = img
                        else:
                            pil_img = tv_functional.to_pil_image(img.cpu())
                        content.append(
                            {
                                "type": "image",
                                "image": pil_img,
                                "min_pixels": 4 * 32 * 32,
                                "max_pixels": 1800 * 32 * 32,
                            }
                        )
                    if contains_text and not contains_image:
                        text = batch["text"][i] if batch["text"][i] else "NULL"
                        content.append({"type": "text", "text": text})

                    conversations.append(
                        [
                            {
                                "role": "system",
                                "content": [
                                    {
                                        "type": "text",
                                        "text": "Represent the user's input.",
                                    }
                                ],
                            },
                            {"role": "user", "content": content},
                        ]
                    )

                text = self._doc_processor.apply_chat_template(
                    conversations,
                    add_generation_prompt=True,
                    tokenize=False,
                )
                try:
                    images, video_inputs, video_kwargs = process_vision_info(
                        conversations,
                        image_patch_size=16,
                        return_video_metadata=True,
                        return_video_kwargs=True,
                    )
                except Exception:
                    images, video_inputs, video_kwargs = (
                        None,
                        None,
                        {"do_sample_frames": False},
                    )

                videos, video_metadata = None, None
                if video_inputs is not None:
                    videos, video_metadata = zip(*video_inputs)
                    videos, video_metadata = list(videos), list(video_metadata)

                processed = self._doc_processor(
                    text=text,
                    images=images,
                    videos=videos,
                    video_metadata=video_metadata,
                    truncation=True,
                    max_length=8192,
                    padding=True,
                    do_resize=False,
                    return_tensors="pt",
                    **video_kwargs,
                )
                processed = {k: v.to(self.device) for k, v in processed.items()}

                outputs = self._doc_model(**processed)

                # Last-token pooling
                attn = processed["attention_mask"]
                flipped = attn.flip(dims=[1])
                last_pos = flipped.argmax(dim=1)
                col = attn.shape[1] - last_pos - 1
                row = torch.arange(
                    outputs.last_hidden_state.shape[0], device=self.device
                )
                embeddings = outputs.last_hidden_state[row, col]
                embeddings = F.normalize(embeddings, p=2, dim=-1)
                all_embeddings.append(embeddings.cpu())

        return torch.cat(all_embeddings, dim=0)

    def encode(
        self,
        inputs: DataLoader[BatchedInput],
        *,
        task_metadata: TaskMetadata,
        hf_split: str,
        hf_subset: str,
        prompt_type: PromptType | None = None,
        show_progress_bar: bool = True,
        **kwargs: Any,
    ) -> Array:
        from mteb.types import PromptType

        # Validate: reject tasks where queries contain images.
        # NanoVDR only supports text-query → image-document retrieval.
        if (
            prompt_type in (PromptType.query, None)
            and "image" in inputs.dataset.features
        ):
            raise ValueError(
                f"NanoVDR only supports text queries, but task "
                f"'{task_metadata.name}' provides image inputs for queries. "
                f"NanoVDR is a text-query → image-document retrieval model "
                f"and does not support image-query or image-classification tasks."
            )

        if prompt_type == PromptType.document:
            return self._encode_documents(inputs, show_progress_bar=show_progress_bar)
        else:
            # Use the lightweight student for queries and all non-retrieval tasks
            return self._encode_queries(inputs, show_progress_bar=show_progress_bar)


nanovdr_s_multi = ModelMeta(
    loader=NanoVDRWrapper,
    name="nanovdr/NanoVDR-S-Multi",
    model_type=["dense"],
    languages=["eng-Latn", "deu-Latn", "fra-Latn", "spa-Latn", "ita-Latn", "por-Latn"],
    open_weights=True,
    revision="b21574d7772ca26e22525543a2a6bf7081a95d8f",
    release_date="2026-02-26",
    modalities=["text", "image"],
    n_parameters=69_000_000,
    n_embedding_parameters=1_572_864,
    memory_usage_mb=282,
    embed_dim=2048,
    license="apache-2.0",
    max_tokens=512,
    reference="https://huggingface.co/nanovdr/NanoVDR-S-Multi",
    similarity_fn_name=ScoringFunction.COSINE,
    framework=["Sentence Transformers", "PyTorch"],
    use_instructions=False,
    public_training_code=None,
    public_training_data="https://huggingface.co/datasets/nanovdr/NanoVDR-Train",
    training_datasets={
        "VidoreTabfquadRetrieval",
        "VidoreDocVQARetrieval",
        "VidoreInfoVQARetrieval",
        "VidoreArxivQARetrieval",
    },
    citation=NANOVDR_CITATION,
)



from mteb.abstasks.retrieval import AbsTaskRetrieval
from mteb.abstasks.task_metadata import TaskMetadata


class SpokenSQuADT2ARetrieval(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="SpokenSQuADT2ARetrieval",
        description="Text-to-audio retrieval task based on SpokenSQuAD dataset. Given a text question, retrieve relevant audio segments that contain the answer. Questions are derived from SQuAD reading comprehension dataset with corresponding spoken passages.",
        reference="https://github.com/chiuwy/Spoken-SQuAD",
        dataset={
            "path": "mteb/spoken-squad-t2a",
            "revision": "b311394fa5ab5e003012304799d0d724d64303b3",
        },
        type="Any2AnyRetrieval",
        category="t2a",
        modalities=["text", "audio"],
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="hit_rate_at_5",
        date=("2018-03-01", "2018-03-01"),
        domains=["Academic", "Encyclopaedic", "Non-fiction"],
        task_subtypes=["Question Answering Retrieval", "Reading Comprehension"],
        license="cc-by-sa-4.0",
        annotations_creators="derived",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@inproceedings{li2018spokensquad,
  author = {Li, Chia-Hsuan and Ma, Szu-Lin and Zhang, Hsin-Wei and Lee, Hung-yi and Lee, Lin-shan},
  booktitle = {Interspeech},
  pages = {3459--3463},
  title = {Spoken SQuAD: A Study of Mitigating the Impact of Speech Recognition Errors on Listening Comprehension},
  year = {2018},
}
""",
    )



from mteb.abstasks import AbsTaskClassification
from mteb.abstasks.task_metadata import TaskMetadata


class Country211Classification(AbsTaskClassification):
    input_column_name: str = "jpg"
    label_column_name: str = "cls"
    samples_per_label: int = 16
    n_experiments: int = 5

    metadata = TaskMetadata(
        name="Country211",
        description="Classifying images of 211 countries.",
        reference="https://huggingface.co/datasets/clip-benchmark/wds_country211",
        dataset={
            "path": "mteb/wds_country211",
            "revision": "c3875324cab4803fa6d3afcd5947a97c84fad4c6",
        },
        type="ImageClassification",
        category="i2c",
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="accuracy",
        date=(
            "2020-01-01",
            "2021-02-26",
        ),  # Estimated range for the collection of reviews
        domains=["Scene"],
        task_subtypes=["Scene recognition"],
        license="cc-by-sa-4.0",
        annotations_creators="derived",
        dialect=[],
        modalities=["image"],
        sample_creation="created",
        bibtex_citation=r"""
@article{radford2021learning,
  author = {Radford, Alec and Kim, Jong Wook and Hallacy, Chris and Ramesh, Aditya and Goh, Gabriel and Agarwal, Sandhini and Sastry, Girish and Askell, Amanda and Mishkin, Pamela and Clark, Jack and others},
  journal = {arXiv preprint arXiv:2103.00020},
  title = {Learning Transferable Visual Models From Natural Language Supervision},
  year = {2021},
}
""",
    )



"""Implementation of Text2Vec models"""

from mteb.models.model_implementations.sentence_transformers_models import (
    sent_trf_training_dataset,
)
from mteb.models.model_meta import ModelMeta, ScoringFunction
from mteb.models.sentence_transformer_wrapper import SentenceTransformerEncoderWrapper

TEXT2VEC_CITATION = """@software{text2vec,
  author = {Xu Ming},
  title = {text2vec: A Tool for Text to Vector},
  year = {2022},
  url = {https://github.com/shibing624/text2vec},
}"""

# I couldn't find the large model on HF for some reason
text2vec_base_chinese = ModelMeta(
    loader=SentenceTransformerEncoderWrapper,
    name="shibing624/text2vec-base-chinese",
    languages=["zho-Hans"],
    open_weights=True,
    revision="183bb99aa7af74355fb58d16edf8c13ae7c5433e",
    release_date="2022-01-23",
    n_parameters=102267648,
    n_embedding_parameters=16_226_304,
    embed_dim=768,
    license="apache-2.0",
    max_tokens=512,
    reference="https://huggingface.co/shibing624/text2vec-base-chinese",
    similarity_fn_name=ScoringFunction.COSINE,
    framework=["Sentence Transformers", "PyTorch", "ONNX", "safetensors"],
    use_instructions=False,
    superseded_by=None,
    adapted_from=None,
    public_training_code=None,
    public_training_data=None,  # Couldn't find it
    training_datasets=set(
        # source: https://huggingface.co/shibing624/text2vec-base-chinese
        # Not in MTEB
        # - shibing624/nli-zh-all/text2vec-base-chinese-sentence-dataset
        # (Could have overlaps I'm not aware of)
    ),
    memory_usage_mb=390,
    citation=TEXT2VEC_CITATION,
)

text2vec_base_chinese_paraphrase = ModelMeta(
    loader=SentenceTransformerEncoderWrapper,
    name="shibing624/text2vec-base-chinese-paraphrase",
    languages=["zho-Hans"],
    open_weights=True,
    revision="e90c150a9c7fb55a67712a766d6820c55fb83cdd",
    release_date="2023-06-19",
    n_parameters=117944832,
    n_embedding_parameters=30_720_000,
    memory_usage_mb=450,
    embed_dim=768,
    license="apache-2.0",
    max_tokens=512,
    reference="https://huggingface.co/shibing624/text2vec-base-chinese-paraphrase",
    similarity_fn_name=ScoringFunction.COSINE,
    framework=["Sentence Transformers", "PyTorch", "safetensors", "Transformers"],
    use_instructions=False,
    superseded_by=None,
    adapted_from=None,
    public_training_code=None,
    public_training_data=None,  # Couldn't find it
    training_datasets=set(
        # source: https://huggingface.co/shibing624/text2vec-base-chinese
        # Not in MTEB
        # - shibing624/nli-zh-all/text2vec-base-chinese-paraphrase
        # (Could have overlaps I'm not aware of)
    ),
    citation=TEXT2VEC_CITATION,
)


text2vec_multi_langs = [
    "deu-Latn",  # German (de)
    "eng-Latn",  # English (en)
    "spa-Latn",  # Spanish (es)
    "fra-Latn",  # French (fr)
    "ita-Latn",  # Italian (it)
    "nld-Latn",  # Dutch (nl)
    "pol-Latn",  # Polish (pl)
    "por-Latn",  # Portuguese (pt)
    "rus-Cyrl",  # Russian (ru)
    "zho-Hans",  # Chinese (Simplified, zh)
]
text2vec_base_multilingual = ModelMeta(
    name="shibing624/text2vec-base-multilingual",
    languages=text2vec_multi_langs,
    open_weights=True,
    revision="6633dc49e554de7105458f8f2e96445c6598e9d1",
    release_date="2023-06-22",
    # While it can be loaded with SBERT, it has one suspicious file according to huggingface
    # So probably best not to.
    loader=SentenceTransformerEncoderWrapper,
    n_parameters=117653760,
    n_embedding_parameters=96_014_208,
    memory_usage_mb=449,
    embed_dim=384,
    license="apache-2.0",
    max_tokens=256,
    reference="https://huggingface.co/shibing624/text2vec-base-chinese-paraphrase",
    similarity_fn_name=ScoringFunction.COSINE,
    framework=[
        "Sentence Transformers",
        "PyTorch",
        "ONNX",
        "safetensors",
        "Transformers",
    ],
    use_instructions=False,
    superseded_by=None,
    adapted_from="sentence-transformers/paraphrase-multilingual-MiniLM-L12-v2",
    public_training_code=None,
    public_training_data=None,  # Couldn't find it
    training_datasets=set(
        # source: https://huggingface.co/shibing624/text2vec-base-chinese
        # Not in MTEB
        # - shibing624/nli-zh-all/tree/main/text2vec-base-multilingual-dataset
        # # (Could have overlaps I'm not aware of)
    )
    | sent_trf_training_dataset,
    citation=TEXT2VEC_CITATION,
)



"""Specific Event Type Models

Defines data structures for various business events
"""

from __future__ import annotations

from typing import Any, Literal

from pydantic import Field

from .base import BaseEvent

# Supported filter types
FilterName = Literal[
    "task_type",
    "domain",
    "modality",
    "language",
    "task",
    "compatibility",
    "availability",
    "instructions",
    "zero-shot",
    "modelParameters",
]


class PageViewEvent(BaseEvent):
    """Page view event

    Triggered when user visits the page
    """

    event_name: str = Field(default="page_view", frozen=True)


class BenchmarkChangeEvent(BaseEvent):
    """Benchmark change event

    Triggered when user switches benchmark in sidebar
    """

    event_name: str = Field(default="benchmark_change", frozen=True)
    properties: dict[str, Any] = Field(
        default_factory=dict, description="Contains old_value and new_value"
    )

    @classmethod
    def create(
        cls, session_id: str, old_value: str | None, new_value: str, **kwargs
    ) -> BenchmarkChangeEvent:
        """Convenience creation method"""
        return cls(
            session_id=session_id,
            benchmark=new_value,
            properties={
                "old_value": old_value,
                "new_value": new_value,
            },
            **kwargs,
        )


class FilterChangeEvent(BaseEvent):
    """Filter change event

    Triggered when user modifies any filter condition
    event_name format: filter_change_{filter_name}
    """

    event_name: str = Field(..., description="filter_change_{filter_name}")
    properties: dict[str, Any] = Field(
        default_factory=dict, description="Contains filter_name, old_value, new_value"
    )

    @classmethod
    def create(
        cls,
        session_id: str,
        filter_name: str,
        new_value: Any,
        old_value: Any = None,
        benchmark: str | None = None,
        filters: dict[str, Any] | None = None,
        **kwargs,
    ) -> FilterChangeEvent:
        """Convenience creation method"""
        return cls(
            event_name=f"filter_change_{filter_name}",
            session_id=session_id,
            benchmark=benchmark,
            filters=filters,
            properties={
                "filter_name": filter_name,
                "old_value": old_value,
                "new_value": new_value,
            },
            **kwargs,
        )


class TableSwitchEvent(BaseEvent):
    """Table switch event

    Triggered when user switches between different table views
    """

    event_name: str = Field(default="table_switch", frozen=True)
    properties: dict[str, Any] = Field(
        default_factory=dict, description="Contains old_table and new_table"
    )

    @classmethod
    def create(
        cls,
        session_id: str,
        old_table: str | None,
        new_table: str,
        benchmark: str | None = None,
        **kwargs,
    ) -> TableSwitchEvent:
        """Convenience creation method"""
        return cls(
            session_id=session_id,
            benchmark=benchmark,
            properties={
                "old_table": old_table,
                "new_table": new_table,
            },
            **kwargs,
        )


class TableDownloadEvent(BaseEvent):
    """Table download event

    Triggered when user downloads table data
    """

    event_name: str = Field(default="table_download", frozen=True)
    properties: dict[str, Any] = Field(
        default_factory=dict,
        description="Contains download-related information, such as format, row_count, etc.",
    )

    @classmethod
    def create(
        cls,
        session_id: str,
        benchmark: str | None = None,
        format: str = "csv",
        row_count: int | None = None,
        **kwargs,
    ) -> TableDownloadEvent:
        """Convenience creation method"""
        props = {"format": format}
        if row_count is not None:
            props["row_count"] = row_count

        return cls(
            session_id=session_id, benchmark=benchmark, properties=props, **kwargs
        )



from __future__ import annotations

from typing import TYPE_CHECKING, Any

import torch
import torch.nn.functional as F
from tqdm.auto import tqdm

from mteb._requires_package import requires_package
from mteb.models.abs_encoder import AbsEncoder
from mteb.models.model_meta import ModelMeta, ScoringFunction

if TYPE_CHECKING:
    from PIL import Image
    from torch.utils.data import DataLoader

    from mteb.abstasks.task_metadata import TaskMetadata
    from mteb.types import Array, BatchedInput, PromptType

NOMIC_EMBED_VISION_CITATION = """@article{nussbaum2024nomicembedvision,
      title={Nomic Embed Vision: Expanding the Latent Space},
      author={Nussbaum, Zach and Duderstadt, Brandon and Mulyar, Andriy},
      journal={arXiv preprint arXiv:2406.18587},
      year={2024},
      eprint={2406.18587},
      archivePrefix={arXiv},
      url={https://arxiv.org/abs/2406.18587}
}"""


class NomicVisionModel(AbsEncoder):
    def __init__(
        self,
        model_name: str,
        revision: str,
        text_model_name: str,
        text_model_revision: str,
        device: str = "cuda" if torch.cuda.is_available() else "cpu",
        **kwargs: Any,
    ):
        from transformers import AutoImageProcessor, AutoModel, AutoTokenizer

        requires_package(self, "einops", model_name, "pip install 'mteb[nomic]'")

        self.vision_model_name = model_name
        self.text_model_name = text_model_name
        self.revision = revision
        self.device = device
        self.processor = AutoImageProcessor.from_pretrained(
            self.vision_model_name, revision=self.revision
        )
        self.vision_model = AutoModel.from_pretrained(
            self.vision_model_name, trust_remote_code=True, revision=self.revision
        ).to(self.device)
        self.text_model = AutoModel.from_pretrained(
            self.text_model_name,
            revision=text_model_revision,
            trust_remote_code=True,
        ).to(self.device)
        self.tokenizer = AutoTokenizer.from_pretrained(
            self.text_model_name, revision=text_model_revision
        )

        self.text_model.eval()
        self.vision_model.eval()

    def preprocess(
        self,
        texts: list[str],
        images: list[Image.Image],
    ):
        return self.processor(
            text=texts, images=images, return_tensors="pt", padding=True
        )

    def get_text_embeddings(
        self,
        texts: DataLoader[BatchedInput],
        show_progress_bar: bool = True,
        **kwargs: Any,
    ):
        all_text_embeddings = []

        with torch.no_grad():
            for batch in tqdm(
                texts, disable=not show_progress_bar, desc="Text Encoding"
            ):
                inputs = self.tokenizer(
                    batch["text"], padding=True, truncation=True, return_tensors="pt"
                )
                inputs = {k: v.to(self.device) for k, v in inputs.items()}
                text_outputs = self.text_model(**inputs)
                text_embeddings = self.mean_pooling(
                    text_outputs, inputs["attention_mask"]
                )
                text_embeddings = F.layer_norm(
                    text_embeddings, normalized_shape=(text_embeddings.shape[1],)
                )
                text_embeddings = F.normalize(text_embeddings, p=2, dim=1)
                all_text_embeddings.append(text_embeddings.cpu())

        all_text_embeddings = torch.cat(all_text_embeddings, dim=0)
        return all_text_embeddings

    def mean_pooling(self, model_output, attention_mask):
        token_embeddings = model_output[0]
        input_mask_expanded = (
            attention_mask.unsqueeze(-1).expand(token_embeddings.size()).float()
        )
        return torch.sum(token_embeddings * input_mask_expanded, 1) / torch.clamp(
            input_mask_expanded.sum(1), min=1e-9
        )

    def get_image_embeddings(
        self,
        images: DataLoader[BatchedInput],
        show_progress_bar: bool = True,
        **kwargs: Any,
    ):
        all_image_embeddings = []

        with torch.no_grad():
            for batch in tqdm(
                images, disable=not show_progress_bar, desc="Image Encoding"
            ):
                inputs = self.processor(images=batch["image"], return_tensors="pt")
                inputs = {k: v.to(self.device) for k, v in inputs.items()}
                image_outputs = self.vision_model(**inputs).last_hidden_state
                img_embeddings = F.normalize(image_outputs[:, 0], p=2, dim=1)
                all_image_embeddings.append(img_embeddings.cpu())
        all_image_embeddings = torch.cat(all_image_embeddings, dim=0)
        return all_image_embeddings

    def encode(
        self,
        inputs: DataLoader[BatchedInput],
        *,
        task_metadata: TaskMetadata,
        hf_split: str,
        hf_subset: str,
        prompt_type: PromptType | None = None,
        **kwargs: Any,
    ) -> Array:
        text_embeddings = None
        image_embeddings = None
        if "text" in inputs.dataset.features:
            text_embeddings = self.get_text_embeddings(inputs, **kwargs)
        if "image" in inputs.dataset.features:
            image_embeddings = self.get_image_embeddings(inputs, **kwargs)

        if text_embeddings is not None and image_embeddings is not None:
            if len(text_embeddings) != len(image_embeddings):
                raise ValueError(
                    "The number of texts and images must have the same length"
                )
            fused_embeddings = text_embeddings + image_embeddings
            return fused_embeddings
        elif text_embeddings is not None:
            return text_embeddings
        elif image_embeddings is not None:
            return image_embeddings
        raise ValueError


nomic_embed_vision_v1_5 = ModelMeta(
    loader=NomicVisionModel,
    loader_kwargs={
        "text_model_name": "nomic-ai/nomic-embed-text-v1.5",
        "text_model_revision": "a03db6748c80237063eb0546ac6b627eca2318cb",
    },
    name="nomic-ai/nomic-embed-vision-v1.5",
    model_type=["dense"],
    languages=["eng-Latn"],
    revision="af2246fffdab78d8458418480e4886a8e48b70a7",
    release_date="2024-06-08",
    modalities=["image", "text"],
    n_parameters=92946688,
    n_embedding_parameters=None,
    memory_usage_mb=355,
    max_tokens=2048,
    embed_dim=768,
    license="apache-2.0",
    open_weights=True,
    public_training_code="https://github.com/nomic-ai/contrastors",
    public_training_data=None,
    framework=["PyTorch", "Transformers", "ONNX", "safetensors"],
    reference="https://huggingface.co/nomic-ai/nomic-embed-vision-v1.5",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=True,
    training_datasets=set(
        # https://arxiv.org/pdf/2406.18587
        # DFN-2B
    ),
    citation=NOMIC_EMBED_VISION_CITATION,
)



from mteb.models.model_meta import ModelMeta, ScoringFunction
from mteb.models.sentence_transformer_wrapper import SentenceTransformerEncoderWrapper

TRAINING_DATA = {
    "MSMARCO",
    "MIRACLRetrieval",
    "LoTTE",
    "TriviaQA",
    "GooAQ",
    "MrTidyRetrieval",
}

CITATION = """@software{Tulkens2025pyNIFE,
  author       = {St\'{e}phan Tulkens},
  title        = {pyNIFE: nearly inference free embeddings in python},
  year         = {2025},
  publisher    = {Zenodo},
  doi          = {10.5281/zenodo.17512919},
  url          = {https://github.com/stephantul/pynife},
  license      = {MIT},
}"""

nife_gte_modernbert_base_as_router_meta = ModelMeta(
    loader=SentenceTransformerEncoderWrapper,  # type: ignore[call-arg]
    loader_kwargs={},
    name="stephantulkens/NIFE-gte-modernbert-base_as_router",
    revision="b038a184acffb51bd9f7002169b2f81575cb74c8",
    release_date="2025-10-30",
    languages=["eng-Latn"],  # assumed from the original model
    n_parameters=225816576,  # query router + corpus router
    n_active_parameters_override=0,  # active parameters is 0 for the query router
    n_embedding_parameters=76802304 + 38682624,  # query router + corpus router
    memory_usage_mb=861,  # both routers together
    max_tokens=8192.0,
    embed_dim=768,
    license="mit",
    open_weights=True,
    public_training_code=None,  # no reproducible training code, but uses https://pypi.org/project/pynife/
    public_training_data="https://huggingface.co/collections/stephantulkens/gte-modernbert-embedpress",  # + its a distilaltion of a model
    framework=["PyTorch", "safetensors", "Sentence Transformers"],
    reference="https://huggingface.co/stephantulkens/NIFE-gte-modernbert-base",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=False,  # assumed
    training_datasets=TRAINING_DATA,
    superseded_by=None,
    modalities=["text"],
    model_type=["router"],
    citation=CITATION,
    contacts=["stephantul"],
    adapted_from="Alibaba-NLP/gte-modernbert-base",
)

nife_mxbai_embed_large_v1_as_router_meta = ModelMeta(
    loader=SentenceTransformerEncoderWrapper,  # type: ignore[call-arg]
    loader_kwargs={},
    name="stephantulkens/NIFE-mxbai-embed-large-v1_as_router",
    revision="de89ee6357e2647d38e607d6ffb82c0139473d02",
    release_date="2025-11-03",
    languages=["eng-Latn"],  # assumed from the original model
    n_parameters=445694976,  # corpus + query router
    n_active_parameters_override=0,  # active parameters is 0 for the query router
    n_embedding_parameters=110553088 + 31_254_528,  # query router + corpus router
    memory_usage_mb=1700,  # both routers together
    max_tokens=512,  # corpus router is trained with max length 512, but the query router / student model is static (so infinite)
    embed_dim=1024,
    license="mit",
    open_weights=True,
    public_training_code=None,  # no reproducible training code, but uses https://pypi.org/project/pynife/
    public_training_data=None,  # its a distilaltion of a model
    framework=["PyTorch", "safetensors", "Sentence Transformers"],
    reference="https://huggingface.co/stephantulkens/NIFE-mxbai-embed-large-v1_as_router",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=False,
    training_datasets=TRAINING_DATA,
    adapted_from="mixedbread-ai/mxbai-embed-large-v1",
    superseded_by=None,
    modalities=["text"],
    model_type=["router"],
    citation=CITATION,
    contacts=["stephantul"],
)



from __future__ import annotations

import logging
from typing import TYPE_CHECKING, Any

import torch
from tqdm.auto import tqdm

from mteb._create_dataloaders import AudioCollator
from mteb._requires_package import requires_audio_dependencies
from mteb.models import ModelMeta
from mteb.models.abs_encoder import AbsEncoder

if TYPE_CHECKING:
    from torch.utils.data import DataLoader

    from mteb import TaskMetadata
    from mteb.types import Array, BatchedInput, PromptType

logger = logging.getLogger(__name__)


class LCOEmbedding(AbsEncoder):
    def __init__(
        self,
        model_name: str,
        revision: str | None = None,
        device: str = "cuda" if torch.cuda.is_available() else "cpu",
        **kwargs: Any,
    ):
        requires_audio_dependencies()
        from transformers import (
            Qwen2_5OmniProcessor,
            Qwen2_5OmniThinkerForConditionalGeneration,
        )

        self.model_name = model_name
        self.device = device
        self.max_audio_length_seconds = 10

        self.processor = Qwen2_5OmniProcessor.from_pretrained(
            model_name, revision=revision
        )
        self.processor.tokenizer.padding_side = "left"

        self.model = Qwen2_5OmniThinkerForConditionalGeneration.from_pretrained(
            model_name, torch_dtype=torch.bfloat16, **kwargs
        ).to(self.device)
        self.model.eval()

        # Audio sampling rate target
        self.sampling_rate = self.processor.feature_extractor.sampling_rate
        # Pre-calculate max samples once
        self.max_samples = int(self.max_audio_length_seconds * self.sampling_rate)

    def encode(
        self,
        inputs: DataLoader[BatchedInput],
        *,
        task_metadata: TaskMetadata,
        hf_split: str,
        hf_subset: str,
        prompt_type: PromptType | None = None,
        show_progress_bar: bool = True,
        **kwargs: Any,
    ) -> Array:
        try:
            from qwen_omni_utils import process_mm_info
        except ImportError:
            raise ImportError(
                "The 'qwen_omni_utils' package is required for this model. "
                "Please install it or ensure it is in your python path."
            )
        all_embeddings = []

        for batch in tqdm(inputs, disable=not show_progress_bar):
            messages_batch = []
            audio_list = batch.get("audio", [])
            text_list = batch.get("text", [])
            batch_size = max(len(audio_list), len(text_list))

            for i in range(batch_size):
                content = []

                audio_row = audio_list[i] if i < len(audio_list) else None

                if audio_row is not None:
                    array = AudioCollator.resample_audio(
                        {"audio": audio_row}, self.sampling_rate, self.max_samples
                    )
                    content.append({"type": "audio", "audio": array})

                text_row = text_list[i] if i < len(text_list) else None
                if text_row is not None:
                    content.append({"type": "text", "text": text_row})

                # Append the training prompt
                prompt_suffix = (
                    "\nSummarize the above audio in one word:"
                    if audio_row
                    else "\nSummarize the above text in one word:"
                )
                content.append({"type": "text", "text": prompt_suffix})

                messages_batch.append([{"role": "user", "content": content}])

            text_prompts = self.processor.apply_chat_template(
                messages_batch, tokenize=False, add_generation_prompt=True
            )

            audio_inputs, _, _ = process_mm_info(
                messages_batch, use_audio_in_video=False
            )

            processor_inputs = self.processor(
                text=text_prompts,
                audio=audio_inputs,
                padding=True,
                return_tensors="pt",
            ).to(self.device)

            with torch.no_grad():
                outputs = self.model(
                    **processor_inputs, output_hidden_states=True, return_dict=True
                )

                embeddings = outputs.hidden_states[-1][:, -1, :]

                all_embeddings.append(embeddings.cpu().to(torch.float32))

        return torch.cat(all_embeddings, dim=0).numpy()


lco_3b = ModelMeta(
    loader=LCOEmbedding,
    name="LCO-Embedding/LCO-Embedding-Omni-3B",
    languages=["eng-Latn"],
    open_weights=True,
    revision="eea763cfaf673e955ae86c64968896a3fea70189",
    release_date="2025-10-23",
    max_tokens=32768,
    n_parameters=4_703_464_448,
    n_embedding_parameters=311164928,
    memory_usage_mb=8978,
    embed_dim=2048,
    license="mit",
    reference="https://huggingface.co/LCO-Embedding/LCO-Embedding-Omni-3B",
    similarity_fn_name="cosine",
    framework=["PyTorch"],
    use_instructions=True,
    public_training_code=None,
    public_training_data=None,
    training_datasets=None,
    modalities=["audio", "text"],
    citation="""
@misc{xiao2025scalinglanguagecentricomnimodalrepresentation,
  title={Scaling Language-Centric Omnimodal Representation Learning},
  author={Chenghao Xiao and Hou Pong Chan and Hao Zhang and Weiwen Xu and Mahani Aljunied and Yu Rong},
  year={2025},
  eprint={2510.11693},
  archivePrefix={arXiv},
  primaryClass={cs.CL},
  url={https://arxiv.org/abs/2510.11693},
}""",
)

lco_7b = ModelMeta(
    loader=LCOEmbedding,
    name="LCO-Embedding/LCO-Embedding-Omni-7B",
    languages=["eng-Latn"],
    open_weights=True,
    revision="3d38f58aae1253a4443b1270b0767f1e533936cf",
    release_date="2025-10-15",
    max_tokens=32768,
    n_parameters=8_931_813_888,
    n_embedding_parameters=544997376,
    memory_usage_mb=17043,
    embed_dim=3584,
    license="mit",
    reference="https://huggingface.co/LCO-Embedding/LCO-Embedding-Omni-7B",
    similarity_fn_name="cosine",
    framework=["PyTorch"],
    use_instructions=True,
    public_training_code=None,
    public_training_data=None,
    training_datasets=None,
    modalities=["audio", "text"],
    citation="""
@misc{xiao2025scalinglanguagecentricomnimodalrepresentation,
  title={Scaling Language-Centric Omnimodal Representation Learning},
  author={Chenghao Xiao and Hou Pong Chan and Hao Zhang and Weiwen Xu and Mahani Aljunied and Yu Rong},
  year={2025},
  eprint={2510.11693},
  archivePrefix={arXiv},
  primaryClass={cs.CL},
  url={https://arxiv.org/abs/2510.11693},
}""",
)



from mteb.abstasks.classification import AbsTaskClassification
from mteb.abstasks.task_metadata import TaskMetadata


class HotelReviewSentimentClassification(AbsTaskClassification):
    metadata = TaskMetadata(
        name="HotelReviewSentimentClassification",
        dataset={
            "path": "mteb/HotelReviewSentimentClassification",
            "revision": "273d5105974460d3979149e29e88c06a8214c541",
        },
        description="HARD is a dataset of Arabic hotel reviews collected from the Booking.com website.",
        reference="https://link.springer.com/chapter/10.1007/978-3-319-67056-0_3",
        type="Classification",
        category="t2c",
        modalities=["text"],
        eval_splits=["train"],
        eval_langs=["ara-Arab"],
        main_score="accuracy",
        date=("2016-06-01", "2016-07-31"),
        domains=["Reviews", "Written"],
        task_subtypes=["Sentiment/Hate speech"],
        license="not specified",
        annotations_creators="derived",
        dialect=["ara-arab-EG", "ara-arab-JO", "ara-arab-LB", "ara-arab-SA"],
        sample_creation="found",
        bibtex_citation=r"""
@article{elnagar2018hotel,
  author = {Elnagar, Ashraf and Khalifa, Yasmin S and Einea, Anas},
  journal = {Intelligent natural language processing: Trends and applications},
  pages = {35--52},
  publisher = {Springer},
  title = {Hotel Arabic-reviews dataset construction for sentiment analysis applications},
  year = {2018},
}
""",
        superseded_by="HotelReviewSentimentClassification.v2",
    )


class HotelReviewSentimentClassificationV2(AbsTaskClassification):
    metadata = TaskMetadata(
        name="HotelReviewSentimentClassification.v2",
        dataset={
            "path": "mteb/HotelReviewSentimentClassification",
            "revision": "f5e6a24acbed4182114ffdf46747090b3f51e836",
        },
        description="HARD is a dataset of Arabic hotel reviews collected from the Booking.com website. This version corrects errors found in the original data. For details, see [pull request](https://github.com/embeddings-benchmark/mteb/pull/2632)",
        reference="https://link.springer.com/chapter/10.1007/978-3-319-67056-0_3",
        type="Classification",
        category="t2c",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["ara-Arab"],
        main_score="accuracy",
        date=("2016-06-01", "2016-07-31"),
        domains=["Reviews", "Written"],
        task_subtypes=["Sentiment/Hate speech"],
        license="not specified",
        annotations_creators="derived",
        dialect=["ara-arab-EG", "ara-arab-JO", "ara-arab-LB", "ara-arab-SA"],
        sample_creation="found",
        bibtex_citation=r"""
@article{elnagar2018hotel,
  author = {Elnagar, Ashraf and Khalifa, Yasmin S and Einea, Anas},
  journal = {Intelligent natural language processing: Trends and applications},
  pages = {35--52},
  publisher = {Springer},
  title = {Hotel Arabic-reviews dataset construction for sentiment analysis applications},
  year = {2018},
}
""",
    )



from mteb.abstasks.task_metadata import TaskMetadata
from mteb.abstasks.text.bitext_mining import AbsTaskBitextMining

TEST_SAMPLES = 2048


class VieMedEVBitextMining(AbsTaskBitextMining):
    metadata = TaskMetadata(
        name="VieMedEVBitextMining",
        dataset={
            "path": "mteb/VieMedEVBitextMining",
            "revision": "56e8a74cdafa10aaceb9fec8272c209b800165de",
        },
        description="A high-quality Vietnamese-English parallel data from the medical domain for machine translation",
        reference="https://aclanthology.org/2015.iwslt-evaluation.11/",
        type="BitextMining",
        category="t2t",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["eng-Latn", "vie-Latn"],
        main_score="f1",
        date=("2024-08-28", "2022-03-28"),
        domains=["Medical", "Written"],
        task_subtypes=[],
        license="cc-by-nc-4.0",  # version is assumed, but was previously unspecified
        annotations_creators="expert-annotated",
        dialect=[],
        sample_creation="human-translated and localized",
        bibtex_citation=r"""
@inproceedings{medev,
  author = {Nhu Vo and Dat Quoc Nguyen and Dung D. Le and Massimo Piccardi and Wray Buntine},
  booktitle = {Proceedings of the 2024 Joint International Conference on Computational Linguistics, Language Resources and Evaluation (LREC-COLING)},
  title = {{Improving Vietnamese-English Medical Machine Translation}},
  year = {2024},
}
""",
    )



from mteb.abstasks.clustering import AbsTaskClustering
from mteb.abstasks.clustering_legacy import AbsTaskClusteringLegacy
from mteb.abstasks.task_metadata import TaskMetadata

_LANGUAGES = {
    "bs": ["bos-Latn"],
    "ca": ["cat-Latn"],
    "cs": ["ces-Latn"],
    "da": ["dan-Latn"],
    "eu": ["eus-Latn"],
    "gv": ["glv-Latn"],
    "ilo": ["ilo-Latn"],
    "ku": ["kur-Latn"],
    "lv": ["lav-Latn"],
    "min": ["min-Latn"],
    "mt": ["mlt-Latn"],
    "sco": ["sco-Latn"],
    "sq": ["sqi-Latn"],
    "wa": ["wln-Latn"],
}


class WikiClusteringP2P(AbsTaskClusteringLegacy):
    metadata = TaskMetadata(
        name="WikiClusteringP2P",
        description="Clustering of wikipedia articles inspired by BlubrbsClusteringP2P. Labels are taken from top-level categories of the respective languages (e.g., https://lv.wikipedia.org/wiki/Kategorija:Pamatkategorijas).",
        reference="https://github.com/Rysias/wiki-clustering",
        dataset={
            "path": "ryzzlestrizzle/multi-wiki-clustering-p2p",
            "revision": "d4d92f8f28be71035be6a96bdfd4e200cf62faa8",
        },
        type="Clustering",
        category="t2c",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=_LANGUAGES,
        main_score="v_measure",
        date=("2001-01-15", "2024-04-15"),
        domains=["Encyclopaedic", "Written"],
        task_subtypes=["Thematic clustering"],
        license="cc-by-sa-3.0",
        annotations_creators="derived",
        dialect=[],
        sample_creation="created",
        bibtex_citation="",  # None exists
        superseded_by="WikiClusteringP2P.v2",
    )


class WikiClusteringFastP2P(AbsTaskClustering):
    max_document_to_embed = 2048
    max_fraction_of_documents_to_embed = None

    metadata = TaskMetadata(
        name="WikiClusteringP2P.v2",
        description="Clustering of wikipedia articles inspired by BlubrbsClusteringP2P. Labels are taken from top-level categories of the respective languages (e.g., https://lv.wikipedia.org/wiki/Kategorija:Pamatkategorijas).",
        reference="https://github.com/Rysias/wiki-clustering",
        dataset={
            "path": "mteb/WikiClusteringP2P.v2",
            "revision": "596293afadbf41fd03571343f9f3d2b869b4f2e3",
        },
        type="Clustering",
        category="t2c",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=_LANGUAGES,
        main_score="v_measure",
        date=("2001-01-15", "2024-04-15"),
        domains=["Encyclopaedic", "Written"],
        task_subtypes=["Thematic clustering"],
        license="cc-by-sa-3.0",
        annotations_creators="derived",
        dialect=[],
        sample_creation="created",
        bibtex_citation="",  # None exists
        adapted_from=["WikiClusteringP2P"],
    )



from collections import defaultdict

from datasets import Dataset, DatasetDict, load_dataset
from tqdm.auto import tqdm

from mteb.abstasks.retrieval import AbsTaskRetrieval
from mteb.abstasks.task_metadata import TaskMetadata


class ClothoA2TRetrieval(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="ClothoA2TRetrieval",
        description="An audio captioning datasetst containing audio clips and their corresponding captions.",
        reference="https://github.com/audio-captioning/clotho-dataset",
        dataset={
            "path": "mteb/Clotho",
            "revision": "c44521cd4067f134e5f5bace4290b59ed773b451",
        },
        type="Any2AnyRetrieval",
        category="a2t",
        modalities=["audio", "text"],
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="hit_rate_at_5",
        date=("2018-01-01", "2018-12-31"),
        domains=["Encyclopaedic", "Written"],
        task_subtypes=["Reasoning as Retrieval"],
        license="mit",
        annotations_creators="derived",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@misc{drossos2019clothoaudiocaptioningdataset,
  archiveprefix = {arXiv},
  author = {Konstantinos Drossos and Samuel Lipping and Tuomas Virtanen},
  eprint = {1910.09387},
  primaryclass = {cs.SD},
  title = {Clotho: An Audio Captioning Dataset},
  url = {https://arxiv.org/abs/1910.09387},
  year = {2019},
}
""",
    )

    def load_data(self, **kwargs):
        if self.data_loaded:
            return

        ds = load_dataset(**self.metadata.dataset, split="test", keep_in_memory=False)

        queries_ds = ds.select_columns(["index", "audio"]).rename_column("index", "id")

        # Corpus: need to split captions, so we build this
        corpus_data = {"id": [], "text": []}
        qrels_dict = defaultdict(dict)

        for row in tqdm(ds, total=len(ds), desc="Loading Clotho AT2 Retrieval Data"):
            index = row["index"]

            for i, text in enumerate(row["text"].split(".")):
                doc_id = f"d-{index}-{i}"
                corpus_data["id"].append(doc_id)
                corpus_data["text"].append(text)
                qrels_dict[index][doc_id] = 1

        self.corpus = DatasetDict({"test": Dataset.from_dict(corpus_data)})
        self.queries = DatasetDict({"test": queries_ds})
        self.relevant_docs = {"test": qrels_dict}
        self.data_loaded = True


class ClothoT2ARetrieval(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="ClothoT2ARetrieval",
        description="An audio captioning datasetst containing audio clips from the Freesound platform and their corresponding captions.",
        reference="https://github.com/audio-captioning/clotho-dataset",
        dataset={
            "path": "mteb/Clotho",
            "revision": "c44521cd4067f134e5f5bace4290b59ed773b451",
        },
        type="Any2AnyRetrieval",
        category="t2a",
        modalities=["text", "audio"],
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="hit_rate_at_5",
        date=("2018-01-01", "2018-12-31"),
        domains=["Encyclopaedic", "Written"],
        task_subtypes=["Reasoning as Retrieval"],
        license="mit",
        annotations_creators="derived",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@misc{drossos2019clothoaudiocaptioningdataset,
  archiveprefix = {arXiv},
  author = {Konstantinos Drossos and Samuel Lipping and Tuomas Virtanen},
  eprint = {1910.09387},
  primaryclass = {cs.SD},
  title = {Clotho: An Audio Captioning Dataset},
  url = {https://arxiv.org/abs/1910.09387},
  year = {2019},
}
""",
    )

    def load_data(self, **kwargs):
        if self.data_loaded:
            return

        ds = load_dataset(**self.metadata.dataset, split="test", keep_in_memory=False)

        # Corpus: reuse dataset with column operations (no copy for audio)
        corpus_ds = ds.select_columns(["index", "audio"]).rename_column("index", "id")

        queries_data = {"id": [], "text": []}
        qrels_dict = defaultdict(dict)

        for row in ds:
            index = row["index"]

            for i, text in enumerate(row["text"].split(".")):
                query_id = f"q-{index}-{i}"
                queries_data["id"].append(query_id)
                queries_data["text"].append(text)
                qrels_dict[query_id][index] = 1

        self.corpus = DatasetDict({"test": corpus_ds})
        self.queries = DatasetDict({"test": Dataset.from_dict(queries_data)})
        self.relevant_docs = {"test": qrels_dict}
        self.data_loaded = True



from mteb.abstasks.classification import AbsTaskClassification
from mteb.abstasks.task_metadata import TaskMetadata


class LccSentimentClassification(AbsTaskClassification):
    metadata = TaskMetadata(
        name="LccSentimentClassification",
        dataset={
            "path": "DDSC/lcc",
            "revision": "de7ba3406ee55ea2cc52a0a41408fa6aede6d3c6",
        },
        description="The leipzig corpora collection, annotated for sentiment",
        reference="https://github.com/fnielsen/lcc-sentiment",
        type="Classification",
        category="t2c",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["dan-Latn"],
        main_score="accuracy",
        date=("2006-01-01", "2006-12-31"),
        domains=["News", "Web", "Written"],
        task_subtypes=["Sentiment/Hate speech"],
        license="cc-by-4.0",
        annotations_creators="expert-annotated",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@inproceedings{quasthoff-etal-2006-corpus,
  address = {Genoa, Italy},
  author = {Quasthoff, Uwe  and
Richter, Matthias  and
Biemann, Christian},
  booktitle = {Proceedings of the Fifth International Conference on Language Resources and Evaluation ({LREC}{'}06)},
  editor = {Calzolari, Nicoletta  and
Choukri, Khalid  and
Gangemi, Aldo  and
Maegaard, Bente  and
Mariani, Joseph  and
Odijk, Jan  and
Tapias, Daniel},
  month = may,
  publisher = {European Language Resources Association (ELRA)},
  title = {Corpus Portal for Search in Monolingual Corpora},
  url = {http://www.lrec-conf.org/proceedings/lrec2006/pdf/641_pdf.pdf},
  year = {2006},
}
""",
        prompt="Classify texts based on sentiment",
    )

    samples_per_label = 16



from .any_sts_evaluator import AnySTSEvaluator
from .clustering_evaluator import ClusteringEvaluator
from .evaluator import Evaluator
from .image.imagetext_pairclassification_evaluator import (
    ImageTextPairClassificationEvaluator,
)
from .pair_classification_evaluator import PairClassificationEvaluator
from .retrieval_evaluator import RetrievalEvaluator
from .sklearn_evaluator import SklearnEvaluator
from .text.bitext_mining_evaluator import BitextMiningEvaluator
from .text.summarization_evaluator import (
    DeprecatedSummarizationEvaluator,
    SummarizationEvaluator,
)
from .zeroshot_classification_evaluator import ZeroShotClassificationEvaluator

__all__ = [
    "AnySTSEvaluator",
    "BitextMiningEvaluator",
    "ClusteringEvaluator",
    "DeprecatedSummarizationEvaluator",
    "Evaluator",
    "ImageTextPairClassificationEvaluator",
    "PairClassificationEvaluator",
    "RetrievalEvaluator",
    "SklearnEvaluator",
    "SummarizationEvaluator",
    "ZeroShotClassificationEvaluator",
]



from mteb.abstasks.retrieval import AbsTaskRetrieval
from mteb.abstasks.task_metadata import TaskMetadata


class ArguAna(AbsTaskRetrieval):
    ignore_identical_ids = True

    metadata = TaskMetadata(
        name="ArguAna",
        description="ArguAna: Retrieval of the Best Counterargument without Prior Topic Knowledge",
        reference="http://argumentation.bplaced.net/arguana/data",
        dataset={
            "path": "mteb/arguana",
            "revision": "c22ab2a51041ffd869aaddef7af8d8215647e41a",
        },
        type="Retrieval",
        category="t2t",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="ndcg_at_10",
        date=["2018-01-01", "2018-07-01"],  # best guess: based on publication date
        domains=["Social", "Web", "Written"],
        task_subtypes=["Discourse coherence"],
        license="cc-by-sa-4.0",
        annotations_creators="derived",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@inproceedings{wachsmuth2018retrieval,
  author = {Wachsmuth, Henning and Syed, Shahbaz and Stein, Benno},
  booktitle = {ACL},
  title = {Retrieval of the Best Counterargument without Prior Topic Knowledge},
  year = {2018},
}
""",
        prompt={"query": "Given a claim, find documents that refute the claim"},
    )



from mteb.abstasks.sts import AbsTaskSTS
from mteb.abstasks.task_metadata import TaskMetadata


class FaroeseSTS(AbsTaskSTS):
    metadata = TaskMetadata(
        name="FaroeseSTS",
        dataset={
            "path": "mteb/FaroeseSTS",
            "revision": "06d4ff0e5366136f3bdce1307b7d220674c45a13",
        },
        description="Semantic Text Similarity (STS) corpus for Faroese.",
        reference="https://aclanthology.org/2023.nodalida-1.74.pdf",
        type="STS",
        category="t2t",
        modalities=["text"],
        eval_splits=["train"],
        eval_langs=["fao-Latn"],
        main_score="cosine_spearman",
        date=("2018-05-01", "2020-03-31"),
        domains=["News", "Web", "Written"],
        task_subtypes=[],
        license="cc-by-4.0",
        annotations_creators="human-annotated",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@inproceedings{snaebjarnarson-etal-2023-transfer,
  address = {Tórshavn, Faroe Islands},
  author = {Snæbjarnarson, Vésteinn  and
Simonsen, Annika  and
Glavaš, Goran  and
Vulić, Ivan},
  booktitle = {Proceedings of the 24th Nordic Conference on Computational Linguistics (NoDaLiDa)},
  month = {may 22--24},
  publisher = {Link{\"o}ping University Electronic Press, Sweden},
  title = {{T}ransfer to a Low-Resource Language via Close Relatives: The Case Study on Faroese},
  year = {2023},
}
""",
    )

    min_score = 0
    max_score = 5



from mteb.abstasks.retrieval import AbsTaskRetrieval
from mteb.abstasks.task_metadata import TaskMetadata


class VizWizIT2TRetrieval(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="VizWizIT2TRetrieval",
        description="Retrieve the correct answer for a question about an image.",
        reference="https://openaccess.thecvf.com/content_cvpr_2018/papers/Gurari_VizWiz_Grand_Challenge_CVPR_2018_paper.pdf",
        dataset={
            "path": "mteb/vizwiz",
            "revision": "c9a6f7bf3f5cbc8e76a29a423ba48d7b549b8d87",
        },
        type="Any2AnyRetrieval",
        category="it2t",
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="ndcg_at_10",
        date=("2018-01-01", "2018-01-01"),
        domains=["Web"],
        task_subtypes=["Image Text Retrieval"],
        license="cc-by-sa-4.0",
        annotations_creators="derived",
        dialect=[],
        modalities=["text", "image"],
        sample_creation="found",
        bibtex_citation=r"""
@inproceedings{gurari2018vizwiz,
  author = {Gurari, Danna and Li, Qing and Stangl, Abigale J and Guo, Anhong and Lin, Chi and Grauman, Kristen and Luo, Jiebo and Bigham, Jeffrey P},
  booktitle = {Proceedings of the IEEE conference on computer vision and pattern recognition},
  pages = {3608--3617},
  title = {Vizwiz grand challenge: Answering visual questions from blind people},
  year = {2018},
}
""",
    )



from mteb.models import SentenceTransformerEncoderWrapper
from mteb.models.model_meta import ModelMeta, ScoringFunction

XLMR_LANGUAGES = [
    "afr-Latn",
    "amh-Latn",
    "ara-Latn",
    "asm-Latn",
    "aze-Latn",
    "bel-Latn",
    "bul-Latn",
    "ben-Latn",
    "ben-Beng",
    "bre-Latn",
    "bos-Latn",
    "cat-Latn",
    "ces-Latn",
    "cym-Latn",
    "dan-Latn",
    "deu-Latn",
    "ell-Latn",
    "eng-Latn",
    "epo-Latn",
    "spa-Latn",
    "est-Latn",
    "eus-Latn",
    "fas-Latn",
    "fin-Latn",
    "fra-Latn",
    "fry-Latn",
    "gle-Latn",
    "gla-Latn",
    "glg-Latn",
    "guj-Latn",
    "hau-Latn",
    "heb-Latn",
    "hin-Latn",
    "hin-Deva",
    "hrv-Latn",
    "hun-Latn",
    "hye-Latn",
    "ind-Latn",
    "isl-Latn",
    "ita-Latn",
    "jpn-Latn",
    "jav-Latn",
    "kat-Latn",
    "kaz-Latn",
    "khm-Latn",
    "kan-Latn",
    "kor-Latn",
    "kur-Latn",
    "kir-Latn",
    "lat-Latn",
    "lao-Latn",
    "lit-Latn",
    "lav-Latn",
    "mlg-Latn",
    "mkd-Latn",
    "mal-Latn",
    "mon-Latn",
    "mar-Latn",
    "msa-Latn",
    "mya-Latn",
    "nep-Latn",
    "nld-Latn",
    "nob-Latn",
    "orm-Latn",
    "ori-Latn",
    "pan-Latn",
    "pol-Latn",
    "pus-Latn",
    "por-Latn",
    "ron-Latn",
    "rus-Latn",
    "san-Latn",
    "snd-Latn",
    "sin-Latn",
    "slk-Latn",
    "slv-Latn",
    "som-Latn",
    "sqi-Latn",
    "srp-Latn",
    "sun-Latn",
    "swe-Latn",
    "swa-Latn",
    "tam-Latn",
    "tam-Taml",
    "tel-Latn",
    "tel-Telu",
    "tha-Latn",
    "tgl-Latn",
    "tur-Latn",
    "uig-Latn",
    "ukr-Latn",
    "urd-Latn",
    "urd-Arab",
    "uzb-Latn",
    "vie-Latn",
    "xho-Latn",
    "yid-Latn",
    "zho-Hant",
    "zho-Hans",
]


xlmr_base = ModelMeta(
    loader=SentenceTransformerEncoderWrapper,  # type: ignore[arg-type]
    name="FacebookAI/xlm-roberta-base",
    model_type=["dense"],
    languages=XLMR_LANGUAGES,
    open_weights=True,
    revision="e73636d4f797dec63c3081bb6ed5c7b0bb3f2089",
    release_date="2019-11-05",  # arxiv paper release
    n_parameters=278043648,
    n_embedding_parameters=192_001_536,
    memory_usage_mb=1064,
    embed_dim=768,
    license="mit",
    max_tokens=512,
    reference="https://huggingface.co/FacebookAI/xlm-roberta-base",
    similarity_fn_name=ScoringFunction.COSINE,
    framework=[
        "Sentence Transformers",
        "PyTorch",
        "Transformers",
        "ONNX",
        "safetensors",
    ],
    use_instructions=False,
    public_training_code=None,
    public_training_data=None,
    training_datasets=set(),
    citation="""@article{DBLP:journals/corr/abs-1911-02116,
  author    = {Alexis Conneau and
               Kartikay Khandelwal and
               Naman Goyal and
               Vishrav Chaudhary and
               Guillaume Wenzek and
               Francisco Guzm{\'{a}}n and
               Edouard Grave and
               Myle Ott and
               Luke Zettlemoyer and
               Veselin Stoyanov},
  title     = {Unsupervised Cross-lingual Representation Learning at Scale},
  journal   = {CoRR},
  volume    = {abs/1911.02116},
  year      = {2019},
  url       = {http://arxiv.org/abs/1911.02116},
  eprinttype = {arXiv},
  eprint    = {1911.02116},
  timestamp = {Mon, 11 Nov 2019 18:38:09 +0100},
  biburl    = {https://dblp.org/rec/journals/corr/abs-1911-02116.bib},
  bibsource = {dblp computer science bibliography, https://dblp.org}
}""",
)

xlmr_large = ModelMeta(
    loader=SentenceTransformerEncoderWrapper,  # type: ignore[arg-type]
    name="FacebookAI/xlm-roberta-large",
    model_type=["dense"],
    languages=XLMR_LANGUAGES,
    open_weights=True,
    revision="c23d21b0620b635a76227c604d44e43a9f0ee389",
    release_date="2019-11-05",  # arxiv paper release
    n_parameters=559890432,
    n_embedding_parameters=256_002_048,
    memory_usage_mb=2141,
    embed_dim=1024,
    license="mit",
    max_tokens=512,
    reference="https://huggingface.co/FacebookAI/xlm-roberta-large",
    similarity_fn_name=ScoringFunction.COSINE,
    framework=[
        "Sentence Transformers",
        "PyTorch",
        "Transformers",
        "ONNX",
        "safetensors",
    ],
    use_instructions=False,
    public_training_code=None,
    public_training_data=None,
    training_datasets=set(),
    citation="""@article{DBLP:journals/corr/abs-1911-02116,
  author    = {Alexis Conneau and
               Kartikay Khandelwal and
               Naman Goyal and
               Vishrav Chaudhary and
               Guillaume Wenzek and
               Francisco Guzm{\'{a}}n and
               Edouard Grave and
               Myle Ott and
               Luke Zettlemoyer and
               Veselin Stoyanov},
  title     = {Unsupervised Cross-lingual Representation Learning at Scale},
  journal   = {CoRR},
  volume    = {abs/1911.02116},
  year      = {2019},
  url       = {http://arxiv.org/abs/1911.02116},
  eprinttype = {arXiv},
  eprint    = {1911.02116},
  timestamp = {Mon, 11 Nov 2019 18:38:09 +0100},
  biburl    = {https://dblp.org/rec/journals/corr/abs-1911-02116.bib},
  bibsource = {dblp computer science bibliography, https://dblp.org}
}""",
)



from mteb.abstasks.retrieval import AbsTaskRetrieval
from mteb.abstasks.task_metadata import TaskMetadata


class GovReportRetrieval(AbsTaskRetrieval):
    metadata = TaskMetadata(
        dataset={
            "path": "isaacus/mteb-GovReport",
            "revision": "4482db2a053706c0aef0ab7bf1878d29bb0295f8",
        },
        name="GovReport",
        description="A dataset for evaluating the ability of information retrieval models to retrieve lengthy US government reports from their summaries.",
        reference="https://huggingface.co/datasets/launch/gov_report",
        type="Retrieval",
        category="t2t",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="ndcg_at_10",
        date=("2021-06-06", "2025-07-28"),
        domains=["Legal", "Government"],
        task_subtypes=[],
        license="cc-by-4.0",
        annotations_creators="expert-annotated",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@inproceedings{huang-etal-2021-efficient,
  address = {Online},
  author = {Huang, Luyang  and
Cao, Shuyang  and
Parulian, Nikolaus  and
Ji, Heng  and
Wang, Lu},
  booktitle = {Proceedings of the 2021 Conference of the North American Chapter of the Association for Computational Linguistics: Human Language Technologies},
  doi = {10.18653/v1/2021.naacl-main.112},
  eprint = {2104.02112},
  month = jun,
  pages = {1419--1436},
  publisher = {Association for Computational Linguistics},
  title = {Efficient Attentions for Long Document Summarization},
  url = {https://aclanthology.org/2021.naacl-main.112},
  year = {2021},
}
""",
    )



from mteb.abstasks.pair_classification import AbsTaskPairClassification
from mteb.abstasks.task_metadata import TaskMetadata


class FarsTail(AbsTaskPairClassification):
    metadata = TaskMetadata(
        name="FarsTail",
        dataset={
            "path": "mteb/FarsTail",
            "revision": "0fa0863dc160869b5a2d78803b4440ea3c671ff5",
        },
        description="This dataset, named FarsTail, includes 10,367 samples which are provided in both the Persian language as well as the indexed format to be useful for non-Persian researchers. The samples are generated from 3,539 multiple-choice questions with the least amount of annotator interventions in a way similar to the SciTail dataset",
        reference="https://link.springer.com/article/10.1007/s00500-023-08959-3",
        type="PairClassification",
        category="t2t",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["fas-Arab"],
        main_score="max_ap",
        date=("2021-01-01", "2021-07-12"),  # best guess
        domains=["Academic", "Written"],
        task_subtypes=["Textual Entailment"],
        license="not specified",
        annotations_creators="human-annotated",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@article{amirkhani2023farstail,
  author = {Amirkhani, Hossein and AzariJafari, Mohammad and Faridan-Jahromi, Soroush and Kouhkan, Zeinab and Pourjafari, Zohreh and Amirak, Azadeh},
  doi = {10.1007/s00500-023-08959-3},
  journal = {Soft Computing},
  publisher = {Springer},
  title = {FarsTail: a Persian natural language inference dataset},
  year = {2023},
}
""",  # after removing neutral
    )



from mteb.abstasks.retrieval import AbsTaskRetrieval
from mteb.abstasks.task_metadata import TaskMetadata


class Fashion200kI2TRetrieval(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="Fashion200kI2TRetrieval",
        description="Retrieve clothes based on descriptions.",
        reference="https://openaccess.thecvf.com/content_iccv_2017/html/Han_Automatic_Spatially-Aware_Fashion_ICCV_2017_paper.html",
        dataset={
            "path": "mteb/mbeir_fashion200k_task3",
            "revision": "74e9e63bf8f802f8cfb4ba6e01f48efc4f49032c",
        },
        type="Any2AnyRetrieval",
        category="i2t",
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="ndcg_at_10",
        date=("2017-01-01", "2017-12-31"),
        domains=["Encyclopaedic"],
        task_subtypes=["Image Text Retrieval"],
        license="apache-2.0",
        annotations_creators="derived",
        dialect=[],
        modalities=["text", "image"],
        sample_creation="created",
        bibtex_citation=r"""
@inproceedings{han2017automatic,
  author = {Han, Xintong and Wu, Zuxuan and Huang, Phoenix X and Zhang, Xiao and Zhu, Menglong and Li, Yuan and Zhao, Yang and Davis, Larry S},
  booktitle = {Proceedings of the IEEE international conference on computer vision},
  pages = {1463--1471},
  title = {Automatic spatially-aware fashion concept discovery},
  year = {2017},
}
""",
        prompt={
            "query": "Based on the following fashion description, retrieve the best matching image."
        },
    )



from mteb.abstasks.task_metadata import TaskMetadata
from mteb.abstasks.text.bitext_mining import AbsTaskBitextMining

_LANGUAGES = {
    "de-en": ["deu-Latn", "eng-Latn"],
    "fr-en": ["fra-Latn", "eng-Latn"],
    "ru-en": ["rus-Cyrl", "eng-Latn"],
    "zh-en": ["cmn-Hans", "eng-Latn"],
}


_SPLITS = ["test"]


class BUCCBitextMiningFast(AbsTaskBitextMining):
    fast_loading = True
    metadata = TaskMetadata(
        name="BUCC.v2",
        dataset={
            "path": "mteb/bucc-bitext-mining",
            "revision": "1739dc11ffe9b7bfccd7f3d585aeb4c544fc6677",
        },
        description="BUCC bitext mining dataset train split, gold set only.",
        reference="https://comparable.limsi.fr/bucc2018/bucc2018-task.html",
        type="BitextMining",
        category="t2t",
        modalities=["text"],
        eval_splits=_SPLITS,
        eval_langs=_LANGUAGES,
        main_score="f1",
        date=("2017-01-01", "2018-12-31"),
        domains=["Written"],
        task_subtypes=[],
        license="not specified",
        annotations_creators="human-annotated",
        dialect=[],
        sample_creation="human-translated",
        bibtex_citation=r"""
@inproceedings{zweigenbaum-etal-2017-overview,
  address = {Vancouver, Canada},
  author = {Zweigenbaum, Pierre  and
Sharoff, Serge  and
Rapp, Reinhard},
  booktitle = {Proceedings of the 10th Workshop on Building and Using Comparable Corpora},
  doi = {10.18653/v1/W17-2512},
  editor = {Sharoff, Serge  and
Zweigenbaum, Pierre  and
Rapp, Reinhard},
  month = aug,
  pages = {60--67},
  publisher = {Association for Computational Linguistics},
  title = {Overview of the Second {BUCC} Shared Task: Spotting Parallel Sentences in Comparable Corpora},
  url = {https://aclanthology.org/W17-2512},
  year = {2017},
}
""",
        adapted_from=["BUCC"],
    )



from mteb.abstasks.retrieval import AbsTaskRetrieval
from mteb.abstasks.task_metadata import TaskMetadata


class RARbCode(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="RARbCode",
        description="Measuring the ability to retrieve the groundtruth answers to reasoning task queries on RAR-b code-pooled dataset.",
        reference="https://arxiv.org/abs/2404.06347",
        dataset={
            "path": "mteb/RARbCode",
            "revision": "ed0a01c3e4d39ec17badb87405e595322f8f2b86",
        },
        type="Retrieval",
        category="t2t",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="ndcg_at_10",
        date=("2019-01-01", "2023-12-31"),
        domains=["Programming", "Written"],
        task_subtypes=["Reasoning as Retrieval"],
        license="cc-by-nc-sa-4.0",
        annotations_creators="derived",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@article{husain2019codesearchnet,
  author = {Husain, Hamel and Wu, Ho-Hsiang and Gazit, Tiferet and Allamanis, Miltiadis and Brockschmidt, Marc},
  journal = {arXiv preprint arXiv:1909.09436},
  title = {{CodeSearchNet} challenge: Evaluating the state of semantic code search},
  year = {2019},
}

@article{muennighoff2023octopack,
  author = {Muennighoff, Niklas and Liu, Qian and Zebaze, Armel and Zheng, Qinkai and Hui, Binyuan and Zhuo, Terry Yue and Singh, Swayam and Tang, Xiangru and Von Werra, Leandro and Longpre, Shayne},
  journal = {arXiv preprint arXiv:2308.07124},
  title = {Octopack: Instruction tuning code large language models},
  year = {2023},
}

@article{xiao2024rar,
  author = {Xiao, Chenghao and Hudson, G Thomas and Moubayed, Noura Al},
  journal = {arXiv preprint arXiv:2404.06347},
  title = {RAR-b: Reasoning as Retrieval Benchmark},
  year = {2024},
}
""",
        prompt={"query": "Retrieve the answer for the following coding problem."},
    )



from mteb.abstasks.retrieval import AbsTaskRetrieval
from mteb.abstasks.task_metadata import TaskMetadata


class IFIRFire(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="IFIRFire",
        dataset={
            "path": "if-ir/fire",
            "revision": "0efa7b1",
        },
        description="Benchmark IFIR fire subset within instruction following abilities. The instructions simulate lawyers' or legal assistants' nuanced queries to retrieve relevant legal documents. ",
        reference="https://arxiv.org/abs/2503.04644",
        type="InstructionRetrieval",
        category="t2t",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="ndcg_at_20",
        date=["2024-05-01", "2024-10-16"],
        domains=["Written", "Legal"],
        task_subtypes=["Article retrieval"],
        license="mit",
        annotations_creators="human-annotated",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@inproceedings{song2025ifir,
  author = {Song, Tingyu and Gan, Guo and Shang, Mingsheng and Zhao, Yilun},
  booktitle = {Proceedings of the 2025 Conference of the Nations of the Americas Chapter of the Association for Computational Linguistics: Human Language Technologies (Volume 1: Long Papers)},
  pages = {10186--10204},
  title = {IFIR: A Comprehensive Benchmark for Evaluating Instruction-Following in Expert-Domain Information Retrieval},
  year = {2025},
}
""",
    )



from mteb.abstasks.retrieval import AbsTaskRetrieval
from mteb.abstasks.task_metadata import TaskMetadata


class RuBQReranking(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="RuBQReranking",
        dataset={
            "path": "mteb/RuBQReranking",
            "revision": "e8233e2234f8b24ab47f203b69d1161c3c0bc5a1",
        },
        description="Paragraph reranking based on RuBQ 2.0. Give paragraphs that answer the question higher scores.",
        reference="https://openreview.net/pdf?id=P5UQFFoQ4PJ",
        type="Reranking",
        category="t2t",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["rus-Cyrl"],
        main_score="map_at_1000",
        date=("2001-01-01", "2021-01-01"),
        domains=["Encyclopaedic", "Written"],
        task_subtypes=[],
        license="cc-by-sa-4.0",
        annotations_creators="human-annotated",
        dialect=[],
        sample_creation="created",
        bibtex_citation=r"""
@inproceedings{RuBQ2021,
  author = {Ivan Rybin and Vladislav Korablinov and Pavel Efimov and Pavel Braslavski},
  booktitle = {ESWC},
  pages = {532--547},
  title = {RuBQ 2.0: An Innovated Russian Question Answering Dataset},
  year = {2021},
}
""",
        prompt={
            "query": "Given a question, retrieve Wikipedia passages that answer the question.",
        },
    )



"""
MTEB ModelMeta for ManiacLabs/miniac-embed.

LEAF-distilled embedding model: E5-small-unsupervised backbone, mxbai-embed-large teacher.
"""

from mteb.models import ModelMeta, SentenceTransformerEncoderWrapper

MINIAC_EMBED_TRAINING_DATASETS = {
    "fineweb",
    "cc_news",
    "english-words-definitions",
    "amazon-qa",
    "MSMARCO",
    "PubMedQA",
    "trivia_qa",
    "LoTTE",
}

miniac_embed = ModelMeta(
    loader=SentenceTransformerEncoderWrapper,
    loader_kwargs=dict(),
    name="ManiacLabs/miniac-embed",
    model_type=["dense"],
    revision="43ad9517b300a0af652a339d4f5f83c1bc36e255",
    release_date="2026-02-19",
    languages=["eng-Latn"],
    open_weights=True,
    framework=[
        "PyTorch",
        "Sentence Transformers",
    ],
    n_parameters=33_360_000,
    n_embedding_parameters=11_917_056,
    memory_usage_mb=127,
    max_tokens=512,
    embed_dim=1024,
    license="apache-2.0",
    reference="https://huggingface.co/ManiacLabs/miniac-embed",
    similarity_fn_name="cosine",
    use_instructions=True,
    adapted_from="intfloat/e5-small-unsupervised",
    superseded_by=None,
    public_training_code=None,
    public_training_data=None,
    training_datasets=MINIAC_EMBED_TRAINING_DATASETS,
    citation=None,
)



from .biosses_sts import BiossesSTS
from .humesickr import HUMESICKR
from .humests12 import HUMESTS12
from .humests_benchmark import HUMESTSBenchmark
from .sickr_sts import SickrSTS
from .sts12_sts import STS12STS
from .sts12_visual_sts import STS12VisualSTS
from .sts13_sts import STS13STS
from .sts13_visual_sts import STS13VisualSTS
from .sts14_sts import STS14STS
from .sts14_visual_sts import STS14VisualSTS
from .sts15_sts import STS15STS
from .sts15_visual_sts import STS15VisualSTS
from .sts16_sts import STS16STS
from .sts16_visual_sts import STS16VisualSTS
from .sts_benchmark_sts import STSBenchmarkSTS

__all__ = [
    "HUMESICKR",
    "HUMESTS12",
    "STS12STS",
    "STS13STS",
    "STS14STS",
    "STS15STS",
    "STS16STS",
    "BiossesSTS",
    "HUMESTSBenchmark",
    "STS12VisualSTS",
    "STS13VisualSTS",
    "STS14VisualSTS",
    "STS15VisualSTS",
    "STS16VisualSTS",
    "STSBenchmarkSTS",
    "SickrSTS",
]



from __future__ import annotations

from typing import TYPE_CHECKING, Any

import torch
from torch.nn.functional import normalize
from tqdm.auto import tqdm

from mteb.models.abs_encoder import AbsEncoder
from mteb.models.model_meta import ModelMeta, ScoringFunction

if TYPE_CHECKING:
    from torch.utils.data import DataLoader

    from mteb.abstasks.task_metadata import TaskMetadata
    from mteb.types import Array, BatchedInput, PromptType

BLIP_CITATION = """@misc{https://doi.org/10.48550/arxiv.2201.12086,
    doi = {10.48550/ARXIV.2201.12086},
    url = {https://arxiv.org/abs/2201.12086},
    author = {Li, Junnan and Li, Dongxu and Xiong, Caiming and Hoi, Steven},
    keywords = {Computer Vision and Pattern Recognition (cs.CV), FOS: Computer and information sciences, FOS: Computer and information sciences},
    title = {BLIP: Bootstrapping Language-Image Pre-training for Unified Vision-Language Understanding and Generation},
    publisher = {arXiv},
    year = {2022},
    copyright = {Creative Commons Attribution 4.0 International}
}"""


class BLIPModel(AbsEncoder):
    def __init__(
        self,
        model_name: str,
        revision: str,
        device: str = "cuda" if torch.cuda.is_available() else "cpu",
        **kwargs: Any,
    ):
        from transformers import BlipForImageTextRetrieval, BlipProcessor

        self.model_name = model_name
        self.device = device
        self.model = BlipForImageTextRetrieval.from_pretrained(
            model_name, revision=revision
        ).to(self.device)
        self.processor = BlipProcessor.from_pretrained(model_name, revision=revision)

    def get_text_embeddings(
        self,
        texts: DataLoader[BatchedInput],
        show_progress_bar: bool = True,
        **kwargs: Any,
    ):
        all_text_embeddings = []

        with torch.no_grad():
            for batch in tqdm(
                texts, disable=not show_progress_bar, desc="Text Encoding"
            ):
                inputs = self.processor(
                    text=batch["text"],
                    return_tensors="pt",
                    padding=True,
                    truncation=True,
                )
                inputs = {k: v.to(self.device) for k, v in inputs.items()}
                # different to CLIPModelWrapper: text_encoder instead of get_text_features and apply projection and normalization
                text_outputs = self.model.text_encoder(**inputs)
                text_outputs = text_outputs[0]
                text_outputs = normalize(
                    self.model.text_proj(text_outputs[:, 0, :]), dim=-1
                )
                all_text_embeddings.append(text_outputs.cpu())

        all_text_embeddings = torch.cat(all_text_embeddings, dim=0)
        return all_text_embeddings

    def get_image_embeddings(
        self,
        images: DataLoader[BatchedInput],
        show_progress_bar: bool = True,
        **kwargs: Any,
    ):
        all_image_embeddings = []

        with torch.no_grad():
            for batch in tqdm(
                images, disable=not show_progress_bar, desc="Image Encoding"
            ):
                inputs = self.processor(
                    images=batch["image"], return_tensors="pt", padding=True
                )
                inputs = {k: v.to(self.device) for k, v in inputs.items()}
                image_outputs = self.model.vision_model(**inputs)
                image_outputs = image_outputs[0]
                image_outputs = normalize(
                    self.model.vision_proj(image_outputs[:, 0, :]), dim=-1
                )
                all_image_embeddings.append(image_outputs.cpu())

        all_image_embeddings = torch.cat(all_image_embeddings, dim=0)
        return all_image_embeddings

    def encode(
        self,
        inputs: DataLoader[BatchedInput],
        *,
        task_metadata: TaskMetadata,
        hf_split: str,
        hf_subset: str,
        prompt_type: PromptType | None = None,
        **kwargs: Any,
    ) -> Array:
        text_embeddings = None
        image_embeddings = None
        if "text" in inputs.dataset.features:
            text_embeddings = self.get_text_embeddings(inputs, **kwargs)
        if "image" in inputs.dataset.features:
            image_embeddings = self.get_image_embeddings(inputs, **kwargs)

        if text_embeddings is not None and image_embeddings is not None:
            if len(text_embeddings) != len(image_embeddings):
                raise ValueError(
                    "The number of texts and images must have the same length"
                )
            fused_embeddings = text_embeddings + image_embeddings
            return fused_embeddings
        elif text_embeddings is not None:
            return text_embeddings
        elif image_embeddings is not None:
            return image_embeddings
        raise ValueError


# in descending order of usage (downloads from huggingface)
blip_image_captioning_large = ModelMeta(
    loader=BLIPModel,
    name="Salesforce/blip-image-captioning-large",
    model_type=["dense"],
    languages=["eng-Latn"],
    revision="2227ac38c9f16105cb0412e7cab4759978a8fd90",
    release_date="2023-12-07",
    modalities=["image", "text"],
    n_parameters=446128642,
    n_embedding_parameters=23_442_432,
    memory_usage_mb=1792,
    max_tokens=512,
    embed_dim=768,
    license="bsd-3-clause",
    open_weights=True,
    public_training_code="https://github.com/salesforce/BLIP",
    public_training_data="https://github.com/salesforce/BLIP",
    framework=["PyTorch", "Transformers", "safetensors"],
    reference="https://huggingface.co/Salesforce/blip-image-captioning-large",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=False,
    training_datasets=set(
        # COCO
        # CC3M+CC12M+SBU
        # LAION115M
    ),
    citation=BLIP_CITATION,
)

blip_image_captioning_base = ModelMeta(
    loader=BLIPModel,
    name="Salesforce/blip-image-captioning-base",
    model_type=["dense"],
    languages=["eng-Latn"],
    revision="89b09ea1789f7addf2f6d6f0dfc4ce10ab58ef84",
    release_date="2023-08-01",
    modalities=["image", "text"],
    n_parameters=224726017,
    n_embedding_parameters=23_442_432,
    memory_usage_mb=942,
    max_tokens=512,
    embed_dim=768,
    license="bsd-3-clause",
    open_weights=True,
    public_training_code="https://github.com/salesforce/BLIP",
    public_training_data="https://github.com/salesforce/BLIP",
    framework=["PyTorch", "Transformers"],
    reference="https://huggingface.co/Salesforce/blip-image-captioning-base",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=False,
    training_datasets=set(
        # COCO
        # CC3M+CC12M+SBU
        # LAION115M
    ),
    citation=BLIP_CITATION,
)


blip_vqa_base = ModelMeta(
    loader=BLIPModel,
    name="Salesforce/blip-vqa-base",
    model_type=["dense"],
    languages=["eng-Latn"],
    revision="c7df8e7cd7aa2ee9af18f56e2b29e59a92651b64",
    release_date="2023-12-07",
    modalities=["image", "text"],
    n_parameters=384672572,
    n_embedding_parameters=23_442_432,
    memory_usage_mb=1467,
    max_tokens=512,
    embed_dim=768,
    license="bsd-3-clause",
    open_weights=True,
    public_training_code="https://github.com/salesforce/BLIP",
    public_training_data="https://github.com/salesforce/BLIP",
    framework=["PyTorch", "Transformers", "safetensors"],
    reference="https://huggingface.co/Salesforce/blip-vqa-base",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=False,
    training_datasets=set(
        # CC3M+CC12M+SBU
        # LAION115M
    ),
    citation=BLIP_CITATION,
)

blip_vqa_capfilt_large = ModelMeta(
    loader=BLIPModel,
    name="Salesforce/blip-vqa-capfilt-large",
    model_type=["dense"],
    languages=["eng-Latn"],
    revision="e53f95265aeab69013fabb5380500ab984adbbb4",
    release_date="2023-01-22",
    modalities=["image", "text"],
    n_parameters=384672572,
    n_embedding_parameters=23_442_432,
    memory_usage_mb=942,
    max_tokens=512,
    embed_dim=768,
    license="bsd-3-clause",
    open_weights=True,
    public_training_code="https://github.com/salesforce/BLIP",
    public_training_data="https://github.com/salesforce/BLIP",
    framework=["PyTorch", "Transformers"],
    reference="https://huggingface.co/Salesforce/blip-vqa-capfilt-large",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=False,
    training_datasets=set(
        # CC3M+CC12M+SBU
        # LAION115M
    ),
    citation=BLIP_CITATION,
)

blip_itm_base_coco = ModelMeta(
    loader=BLIPModel,
    name="Salesforce/blip-itm-base-coco",
    model_type=["dense"],
    languages=["eng-Latn"],
    revision="7eaa90c11850c0b17fc38c6a11e7d88bd6ac231f",
    release_date="2023-08-01",
    modalities=["image", "text"],
    n_parameters=223744258,
    n_embedding_parameters=23_442_432,
    memory_usage_mb=942,
    max_tokens=512,
    embed_dim=768,
    license="bsd-3-clause",
    open_weights=True,
    public_training_code="https://github.com/salesforce/BLIP",
    public_training_data="https://github.com/salesforce/BLIP",
    framework=["PyTorch", "Transformers"],
    reference="https://huggingface.co/Salesforce/blip-itm-base-coco",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=False,
    training_datasets=set(
        # CC3M+CC12M+SBU
        # LAION115M
    ),
    citation=BLIP_CITATION,
)

blip_itm_large_coco = ModelMeta(
    loader=BLIPModel,
    name="Salesforce/blip-itm-large-coco",
    model_type=["dense"],
    languages=["eng-Latn"],
    revision="fef05cafc05298067cbbca00b125749394a77a6f",
    release_date="2023-08-01",
    modalities=["image", "text"],
    n_parameters=446128642,
    n_embedding_parameters=23_442_432,
    memory_usage_mb=1793,
    max_tokens=512,
    embed_dim=768,
    license="bsd-3-clause",
    open_weights=True,
    public_training_code="https://github.com/salesforce/BLIP",
    public_training_data="https://github.com/salesforce/BLIP",
    framework=["PyTorch", "Transformers"],
    reference="https://huggingface.co/Salesforce/blip-itm-large-coco",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=False,
    training_datasets=set(
        # COCO
        # CC3M+CC12M+SBU
        # LAION115M
    ),
    citation=BLIP_CITATION,
)

blip_itm_base_flickr = ModelMeta(
    loader=BLIPModel,
    name="Salesforce/blip-itm-base-flickr",
    model_type=["dense"],
    languages=["eng-Latn"],
    revision="1de29e660d91ae1786c1876212ea805a22eab251",
    release_date="2023-08-01",
    modalities=["image", "text"],
    n_parameters=223744258,
    n_embedding_parameters=23_442_432,
    memory_usage_mb=942,
    max_tokens=512,
    embed_dim=768,
    license="bsd-3-clause",
    open_weights=True,
    public_training_code="https://github.com/salesforce/BLIP",
    public_training_data="https://github.com/salesforce/BLIP",
    framework=["PyTorch", "Transformers"],
    reference="https://huggingface.co/Salesforce/blip-itm-base-flickr",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=False,
    training_datasets=set(
        # CC3M+CC12M+SBU
        # LAION115M
        # Flickr30k
    ),
    citation=BLIP_CITATION,
)

blip_itm_large_flickr = ModelMeta(
    loader=BLIPModel,
    name="Salesforce/blip-itm-large-flickr",
    model_type=["dense"],
    languages=["eng-Latn"],
    revision="bda12e6506758f54261b5ab174b2c55a3ba143fb",
    release_date="2023-08-01",
    modalities=["image", "text"],
    n_parameters=446128642,
    n_embedding_parameters=23_442_432,
    memory_usage_mb=1793,
    max_tokens=512,
    embed_dim=768,
    license="bsd-3-clause",
    open_weights=True,
    public_training_code="https://github.com/salesforce/BLIP",
    public_training_data="https://github.com/salesforce/BLIP",
    framework=["PyTorch", "Transformers"],
    reference="https://huggingface.co/Salesforce/blip-itm-large-flickr",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=False,
    training_datasets=set(
        # CC3M+CC12M+SBU
        # LAION115M
    ),
    citation=BLIP_CITATION,
)



from mteb.abstasks.retrieval import AbsTaskRetrieval
from mteb.abstasks.task_metadata import TaskMetadata


class AlloprofReranking(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="AlloprofReranking",
        description="This dataset was provided by AlloProf, an organisation in Quebec, Canada offering resources and a help forum curated by a large number of teachers to students on all subjects taught from in primary and secondary school",
        reference="https://huggingface.co/datasets/antoinelb7/alloprof",
        dataset={
            "path": "mteb/AlloprofReranking",
            "revision": "a7d2d793f2e5ba55139bb10088c2e8ee2df2ce02",
        },
        type="Reranking",
        category="t2t",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["fra-Latn"],
        main_score="map_at_1000",
        date=("2020-01-01", "2023-04-14"),  # supposition
        domains=["Web", "Academic", "Written"],
        task_subtypes=["Question answering"],
        license="cc-by-nc-sa-4.0",
        annotations_creators="expert-annotated",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@misc{lef23,
  author = {Lefebvre-Brossard, Antoine and Gazaille, Stephane and Desmarais, Michel C.},
  copyright = {Creative Commons Attribution Non Commercial Share Alike 4.0 International},
  doi = {10.48550/ARXIV.2302.07738},
  keywords = {Computation and Language (cs.CL), Information Retrieval (cs.IR), Machine Learning (cs.LG), FOS: Computer and information sciences, FOS: Computer and information sciences},
  publisher = {arXiv},
  title = {Alloprof: a new French question-answer education dataset and its use in an information retrieval case study},
  url = {https://arxiv.org/abs/2302.07738},
  year = {2023},
}
""",
    )



from mteb.abstasks.retrieval import AbsTaskRetrieval
from mteb.abstasks.task_metadata import TaskMetadata


class VisualNewsT2IRetrieval(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="VisualNewsT2IRetrieval",
        description="Retrieve news images with captions.",
        reference="https://aclanthology.org/2021.emnlp-main.542/",
        dataset={
            "path": "mteb/mbeir_visualnews_task0",
            "revision": "6ba75433fbeb6a06d9fccb27031bcafdfb8b5de1",
        },
        type="Any2AnyRetrieval",
        category="t2i",
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="ndcg_at_10",
        date=("2020-01-01", "2020-12-31"),
        domains=["Encyclopaedic"],
        task_subtypes=["Image Text Retrieval"],
        license="cc-by-sa-4.0",
        annotations_creators="derived",
        dialect=[],
        modalities=["image", "text"],
        sample_creation="created",
        bibtex_citation=r"""
@inproceedings{liu2021visual,
  author = {Liu, Fuxiao and Wang, Yinghan and Wang, Tianlu and Ordonez, Vicente},
  booktitle = {Proceedings of the 2021 Conference on Empirical Methods in Natural Language Processing},
  pages = {6761--6771},
  title = {Visual News: Benchmark and Challenges in News Image Captioning},
  year = {2021},
}
""",
        prompt={
            "query": "Identify the news-related image in line with the described event."
        },
    )



from __future__ import annotations

import logging
from collections import defaultdict
from typing import TYPE_CHECKING, Any, TypedDict

import numpy as np
from datasets import Dataset, DatasetDict
from sklearn.linear_model import LogisticRegression
from sklearn.metrics import (
    accuracy_score,
    average_precision_score,
    f1_score,
    precision_score,
    recall_score,
)
from sklearn.model_selection import KFold

from mteb._create_dataloaders import create_dataloader
from mteb._evaluators.sklearn_evaluator import SklearnEvaluator
from mteb.models import EncoderProtocol
from mteb.types.statistics import (
    SplitDescriptiveStatistics,
)

from ._statistics_calculation import (
    calculate_audio_statistics,
    calculate_image_statistics,
    calculate_label_statistics,
    calculate_text_statistics,
)
from .abstask import AbsTask

if TYPE_CHECKING:
    from pathlib import Path

    from numpy.typing import NDArray

    from mteb._evaluators.sklearn_evaluator import SklearnModelProtocol
    from mteb.models import MTEBModels
    from mteb.types import Array, EncodeKwargs, HFSubset, ScoresDict
    from mteb.types.statistics import (
        AudioStatistics,
        ImageStatistics,
        LabelStatistics,
        TextStatistics,
    )

logger = logging.getLogger(__name__)


class ClassificationDescriptiveStatistics(SplitDescriptiveStatistics):
    """Descriptive statistics for Classification

    Attributes:
        num_samples: number of samples in the dataset.
        number_texts_intersect_with_train: Number of texts in the train split

        text_statistics: Statistics for text
        image_statistics: Statistics for images
        audio_statistics: Statistics for audio
        label_statistics: Statistics for labels
    """

    num_samples: int
    number_texts_intersect_with_train: int | None

    text_statistics: TextStatistics | None
    image_statistics: ImageStatistics | None
    audio_statistics: AudioStatistics | None
    label_statistics: LabelStatistics


class ClassificationMetrics(TypedDict):
    """Scores for classification tasks

    Attributes:
        accuracy: Accuracy score.
        f1: F1 score (macro).
        f1_weighted: Weighted F1 score.
        precision: Precision score (macro).
        precision_weighted: Weighted precision score.
        recall: Recall score (macro).
        recall_weighted: Weighted recall score.
        ap: Average precision score (macro) for binary classification.
        ap_weighted: Weighted average precision score for binary classification.
    """

    accuracy: float
    f1: float
    f1_weighted: float
    precision: float
    precision_weighted: float
    recall: float
    recall_weighted: float
    ap: float | None
    ap_weighted: float | None


class FullClassificationMetrics(ClassificationMetrics):
    """Full classification metrics including scores per experiment. In main scores, the average over all experiments is reported.

    Attributes:
        scores_per_experiment: List of ClassificationMetrics for each experiment.
    """

    scores_per_experiment: list[ClassificationMetrics]


class AbsTaskClassification(AbsTask):
    """The class which classification tasks inherit from.

    A classification task consists of a dataset with input data and corresponding labels. The task is to predict the label for each input.
    The task works by training a sklearn compatible model on samples drawn from the training split of the dataset,
    where the input data is encoded using the provided model.
    The trained model is then evaluated on the evaluation split of the dataset. This process is repeated for `n_experiments` times, and both average and
    individual scores for each experiment are reported.

    Attributes:
        dataset: Hugging Face dataset containing the data for the task. Should have train split (split name can be changed by train_split. Must contain the following columns:
            text: str (for text) or PIL.Image (for image). Column name can be changed via `input_column_name` attribute.
            label: int. Column name can be changed via `label_column_name` attribute.
        evaluator_model: The model to use for evaluation. Can be any sklearn compatible model. Default is `LogisticRegression`.
        samples_per_label: Number of samples per label to use for training the evaluator model. Default is 8.
        n_experiments: Number of experiments to run. Default is 10.
        train_split: Name of the split to use for training the evaluator model. Default is "train".
        label_column_name: Name of the column containing the labels. Default is "label".
        input_column_name: Name of the column containing the input data. Default is "text".
        abstask_prompt: Prompt to use for the task for instruction model if not prompt is provided in TaskMetadata.prompt.
        is_cross_validation: Is task cross validation
        n_splits: Number of splits for cross-validation
    """

    evaluator: type[SklearnEvaluator] = SklearnEvaluator
    evaluator_model: SklearnModelProtocol = LogisticRegression(
        max_iter=100,
    )

    samples_per_label: int = 8
    n_experiments: int = 10
    train_split: str = "train"
    label_column_name: str = "label"
    input_column_name: str = "text"
    abstask_prompt = "Classify user passages."
    is_cross_validation: bool = False
    n_splits = 5

    def evaluate(
        self,
        model: MTEBModels,
        split: str = "test",
        subsets_to_run: list[HFSubset] | None = None,
        *,
        encode_kwargs: EncodeKwargs,
        prediction_folder: Path | None = None,
        num_proc: int | None = None,
        **kwargs: Any,
    ) -> dict[HFSubset, ScoresDict]:
        """Evaluate a model on the classification task.

        Differs from other tasks as it requires train split.
        """
        if not isinstance(model, EncoderProtocol):
            raise TypeError(
                f"Model {model} is a SearchProtocol, but this task {self.metadata.name} does not support Search. "
                "Please use a Encoder model instead."
            )

        if not self.data_loaded:
            self.load_data(num_proc=num_proc)

        if self.dataset is None:
            raise RuntimeError("Dataset not loaded.")

        if "random_state" in self.evaluator_model.get_params():
            self.evaluator_model = self.evaluator_model.set_params(
                random_state=self.seed
            )
        scores = {}
        hf_subsets = self.hf_subsets
        if subsets_to_run is not None:
            hf_subsets = [s for s in hf_subsets if s in subsets_to_run]

        for hf_subset in hf_subsets:
            logger.info(
                f"Task: {self.metadata.name}, split: {split}, subset: {hf_subset}. Running..."
            )

            if hf_subset not in self.dataset and hf_subset == "default":
                ds = self.dataset
            else:
                ds = self.dataset[hf_subset]

            if isinstance(ds, Dataset | DatasetDict):
                ds = ds.select_columns([self.label_column_name, self.input_column_name])
            eval_function = (
                self._evaluate_subset
                if not self.is_cross_validation
                else self._evaluate_subset_cross_validation
            )
            scores[hf_subset] = eval_function(
                model,
                ds,
                hf_split=split,
                hf_subset=hf_subset,
                encode_kwargs=encode_kwargs,
                prediction_folder=prediction_folder,
                num_proc=num_proc,
                **kwargs,
            )
            self._add_main_score(scores[hf_subset])

        return scores  # type: ignore[return-value]

    def _evaluate_subset(
        self,
        model: MTEBModels,
        data_split: DatasetDict,
        *,
        encode_kwargs: EncodeKwargs,
        hf_split: str,
        hf_subset: str,
        prediction_folder: Path | None = None,
        num_proc: int | None = None,
        **kwargs: Any,
    ) -> FullClassificationMetrics:
        if not isinstance(model, EncoderProtocol):
            raise TypeError("Expected model to be an instance of EncoderProtocol")

        train_split = data_split[self.train_split]
        eval_split = data_split[hf_split]

        scores = []
        # we store idxs to make the shuffling reproducible
        test_cache, idxs = None, None

        all_predictions = []
        for i in range(self.n_experiments):
            logger.info(f"Running experiment ({i}/{self.n_experiments})")
            scores_exp, predictions, idxs, test_cache = self._run_experiment(
                model,
                train_split,
                eval_split,
                experiment_num=i,
                idxs=idxs,
                test_cache=test_cache,
                encode_kwargs=encode_kwargs,
                hf_split=hf_split,
                hf_subset=hf_subset,
                num_proc=num_proc,
            )

            if prediction_folder:
                all_predictions.append(predictions)
            scores.append(scores_exp)

        if prediction_folder:
            self._save_task_predictions(
                all_predictions,
                model,
                prediction_folder,
                hf_subset=hf_subset,
                hf_split=hf_split,
            )

        return self._calculate_avg_scores(scores)

    def _evaluate_subset_cross_validation(
        self,
        model: EncoderProtocol,
        data_split: DatasetDict,
        *,
        encode_kwargs: EncodeKwargs,
        hf_split: str,
        hf_subset: str,
        prediction_folder: Path | None = None,
        num_proc: int | None = None,
        **kwargs: Any,
    ) -> FullClassificationMetrics:
        if self.train_split != hf_split:
            raise ValueError(
                f"Performing {self.n_splits}-fold cross validation, but the dataset has a train (`{self.train_split}`) and test split (`{hf_split}`)! Set `is_cross_validation` to False, and retry."
            )
        logger.info(
            f"Performing {self.n_splits}-fold cross-validation on the entire dataset!"
        )

        ds = data_split[self.train_split]
        num_samples = len(ds)

        scores = []
        idxs = None
        cross_validation_splitter = KFold(
            n_splits=self.n_splits, shuffle=True, random_state=self.seed
        )
        all_predictions = []
        dataloader_train = create_dataloader(
            ds,
            self.metadata,
            input_column=self.input_column_name,
            num_proc=num_proc,
            **encode_kwargs,
        )
        logger.info("Running cross-validation - Encoding samples...")
        # precompute all embeddings for cross-validation to not recomupute them in different k-folds
        dataset_embeddings = model.encode(
            dataloader_train,
            task_metadata=self.metadata,
            hf_split=hf_split,
            hf_subset=hf_subset,
            **encode_kwargs,
        )
        for i, (train_idx, val_idx) in enumerate(
            cross_validation_splitter.split(range(num_samples))
        ):
            train_split = ds.select(train_idx)
            eval_split = ds.select(val_idx)
            train_cache = dataset_embeddings[train_idx]
            test_cache = dataset_embeddings[val_idx]
            logger.info(f"Running experiment ({i}/{self.n_experiments})")
            scores_exp, predictions, idxs, _ = self._run_experiment(
                model,
                train_split,
                eval_split,
                experiment_num=i,
                idxs=idxs,
                encode_kwargs=encode_kwargs,
                hf_split=hf_split,
                hf_subset=hf_subset,
                test_cache=test_cache,
                train_cache=train_cache,
                num_proc=num_proc,
            )

            if prediction_folder:
                all_predictions.append(predictions)
            scores.append(scores_exp)

        if prediction_folder:
            self._save_task_predictions(
                all_predictions,
                model,
                prediction_folder,
                hf_subset=hf_subset,
                hf_split=hf_split,
            )
        return self._calculate_avg_scores(scores)

    def _run_experiment(
        self,
        model: EncoderProtocol,
        train_split: Dataset,
        eval_split: Dataset,
        experiment_num: int,
        idxs: list[int] | None,
        test_cache: Array | None,
        *,
        encode_kwargs: EncodeKwargs,
        hf_split: str,
        hf_subset: str,
        train_cache: Array | None = None,
        num_proc: int | None = None,
    ) -> tuple[ClassificationMetrics, list[float], list[int], Array]:
        train_dataset, idxs, selected_idx = self._undersample_data(
            train_split,
            experiment_num,
            idxs,
        )
        sub_train_cache = None
        if train_cache is not None:
            sub_train_cache = train_cache[selected_idx]

        evaluator = self.evaluator(
            train_dataset,
            eval_split,
            self.input_column_name,
            self.label_column_name,
            task_metadata=self.metadata,
            hf_split=hf_split,
            hf_subset=hf_subset,
            evaluator_model=self.evaluator_model,
        )
        y_pred, test_cache = evaluator(
            model,
            encode_kwargs=encode_kwargs,
            test_cache=test_cache,
            train_cache=sub_train_cache,
            num_proc=num_proc,
        )
        y_test = eval_split[self.label_column_name]
        return self._calculate_scores(y_test, y_pred), y_pred.tolist(), idxs, test_cache

    def _calculate_avg_scores(
        self, scores: list[ClassificationMetrics]
    ) -> FullClassificationMetrics:
        avg_scores: dict[str, Any] = {
            # ap will be none for non binary classification tasks
            k: (
                float(np.mean(values))
                if (values := [s[k] for s in scores if s[k] is not None])  # type: ignore[literal-required]
                else np.nan
            )
            for k in scores[0].keys()
        }
        logger.info(f"Running {self.metadata.name} - Finished.")
        return FullClassificationMetrics(
            scores_per_experiment=scores,
            **avg_scores,  # type: ignore[typeddict-item]
        )

    def _calculate_scores(
        self,
        y_test: NDArray[np.integer] | list[int],
        y_pred: NDArray[np.integer | np.floating] | list[int],
    ) -> ClassificationMetrics:
        scores = ClassificationMetrics(
            accuracy=accuracy_score(y_test, y_pred),
            f1=f1_score(y_test, y_pred, average="macro"),
            f1_weighted=f1_score(y_test, y_pred, average="weighted"),
            precision=precision_score(y_test, y_pred, average="macro"),
            precision_weighted=precision_score(y_test, y_pred, average="weighted"),
            recall=recall_score(y_test, y_pred, average="macro"),
            recall_weighted=recall_score(y_test, y_pred, average="weighted"),
            ap=None,
            ap_weighted=None,
        )

        # if binary classification
        if len(np.unique(y_test)) == 2:
            scores["ap"] = average_precision_score(y_test, y_pred, average="macro")
            scores["ap_weighted"] = average_precision_score(
                y_test, y_pred, average="weighted"
            )
        return scores

    def _undersample_data(
        self, dataset: Dataset, experiment_num: int, idxs: list[int] | None = None
    ) -> tuple[Dataset, list[int], list[int]]:
        """Undersample data to have `samples_per_label` samples of each label.

        Args:
            dataset: Hugging Face `datasets.Dataset` containing "text" and "label".
            experiment_num: Experiment number, used to set the random seed.
            idxs: Optional indices to shuffle and sample from.

        Returns:
            Tuple of:
            - A new Dataset containing undersampled examples.
            - The shuffled indices used for sampling.
            - Selected indexes
        """
        if idxs is None:
            idxs = list(range(len(dataset)))

        # using RandomState for backward compatibility with `v1`
        rng_state = np.random.RandomState(self.seed)
        rng_state.shuffle(idxs)

        label_counter: dict[str, int] = defaultdict(int)
        sampled_idxs = []

        for i in idxs:
            label = dataset[i][self.label_column_name]
            if label_counter[label] < self.samples_per_label:
                sampled_idxs.append(i)
                label_counter[label] += 1

        return dataset.select(sampled_idxs), idxs, sampled_idxs

    def _calculate_descriptive_statistics_from_split(
        self, split: str, hf_subset: str | None = None, compute_overall: bool = False
    ) -> ClassificationDescriptiveStatistics:
        train_text = []
        if hf_subset:
            inputs = self.dataset[hf_subset][split][self.input_column_name]
            label = self.dataset[hf_subset][split][self.label_column_name]
            if split != self.train_split:
                train_text = self.dataset[hf_subset][self.train_split][
                    self.input_column_name
                ]
        elif compute_overall:
            inputs = []
            label = []
            for hf_subset in self.metadata.eval_langs:
                inputs.extend(self.dataset[hf_subset][split][self.input_column_name])
                label.extend(self.dataset[hf_subset][split][self.label_column_name])
                if split != self.train_split:
                    train_text.extend(
                        self.dataset[hf_subset][self.train_split][
                            self.input_column_name
                        ]
                    )
        else:
            inputs = self.dataset[split][self.input_column_name]
            label = self.dataset[split][self.label_column_name]
            if split != self.train_split:
                train_text = self.dataset[self.train_split][self.input_column_name]

        image_statistics = None
        text_statistics = None
        audio_statistics = None
        num_texts_in_train = None

        if "image" in self.metadata.modalities:
            image_statistics = calculate_image_statistics(inputs)
        if "text" in self.metadata.modalities:
            text_statistics = calculate_text_statistics(inputs)
            num_texts_in_train = (
                len(set(inputs) & set(train_text))
                if split != self.train_split
                else None
            )
        if "audio" in self.metadata.modalities:
            audio_statistics = calculate_audio_statistics(inputs)

        label_statistics = calculate_label_statistics(label)

        return ClassificationDescriptiveStatistics(
            num_samples=len(inputs),
            number_texts_intersect_with_train=num_texts_in_train,
            text_statistics=text_statistics,
            image_statistics=image_statistics,
            audio_statistics=audio_statistics,
            label_statistics=label_statistics,
        )

    def _push_dataset_to_hub(
        self,
        repo_name: str,
        num_proc: int | None = None,
    ) -> None:
        self._upload_dataset_to_hub(
            repo_name,
            [
                self.input_column_name,
                self.label_column_name,
            ],
            num_proc=num_proc,
        )



from mteb.abstasks.sts import AbsTaskSTS
from mteb.abstasks.task_metadata import TaskMetadata


class KorSTS(AbsTaskSTS):
    metadata = TaskMetadata(
        name="KorSTS",
        dataset={
            "path": "dkoterwa/kor-sts",
            "revision": "016f35f9b961daaaa7a352e927084e3da662ac1f",
        },
        description="Benchmark dataset for STS in Korean. Created by machine translation and human post editing of the STS-B dataset.",
        reference="https://arxiv.org/abs/2004.03289",
        type="STS",
        category="t2t",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["kor-Hang"],
        main_score="cosine_spearman",
        date=("2012-01-01", "2017-01-01"),  # rough approximates
        domains=["News", "Web"],
        task_subtypes=["Textual Entailment"],
        license="cc-by-sa-4.0",
        annotations_creators="human-annotated",
        dialect=[],
        sample_creation="machine-translated and localized",
        bibtex_citation=r"""
@article{ham2020kornli,
  author = {Ham, Jiyeon and Choe, Yo Joong and Park, Kyubyong and Choi, Ilji and Soh, Hyungjoon},
  journal = {arXiv preprint arXiv:2004.03289},
  title = {KorNLI and KorSTS: New Benchmark Datasets for Korean Natural Language Understanding},
  year = {2020},
}
""",
    )

    min_score = 0
    max_score = 5



from __future__ import annotations

import logging
from typing import TYPE_CHECKING, Any

import torch

from mteb.models.abs_encoder import AbsEncoder
from mteb.models.model_meta import ModelMeta, ScoringFunction

if TYPE_CHECKING:
    from collections.abc import Callable

    from torch.utils.data import DataLoader

    from mteb.abstasks.task_metadata import TaskMetadata
    from mteb.models.models_protocols import EncoderProtocol
    from mteb.types import Array, BatchedInput, PromptType

from .repllama_models import RepLLaMAModel, model_prompts

logger = logging.getLogger(__name__)


class PromptrieverModel(RepLLaMAModel, AbsEncoder):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def encode(
        self,
        inputs: DataLoader[BatchedInput],
        *,
        task_metadata: TaskMetadata,
        hf_split: str,
        hf_subset: str,
        prompt_type: PromptType | None = None,
        **kwargs: Any,
    ) -> Array:
        kwargs["is_promptriever"] = True
        return super().encode(
            inputs,
            task_metadata=task_metadata,
            hf_split=hf_split,
            hf_subset=hf_subset,
            prompt_type=prompt_type,
            **kwargs,
        )


def _loader(
    wrapper: type[PromptrieverModel], **kwargs
) -> Callable[..., EncoderProtocol]:
    _kwargs = kwargs

    def loader_inner(**kwargs: Any) -> EncoderProtocol:
        return wrapper(**_kwargs, **kwargs)

    return loader_inner


PROMPTRIEVER_CITATION = """
@article{weller2024promptriever,
      title={Promptriever: Instruction-Trained Retrievers Can Be Prompted Like Language Models},
      author={Orion Weller and Benjamin Van Durme and Dawn Lawrie and Ashwin Paranjape and Yuhao Zhang and Jack Hessel},
      year={2024},
      eprint={2409.11136},
      archivePrefix={arXiv},
      primaryClass={cs.IR},
      url={https://arxiv.org/abs/2409.11136},
}
"""


promptriever_llama2 = ModelMeta(
    loader=_loader(
        PromptrieverModel,
        base_model_name_or_path="meta-llama/Llama-2-7b-hf",
        peft_model_name_or_path="samaya-ai/promptriever-llama2-7b-v1",
        device_map="auto",
        torch_dtype=torch.bfloat16,
        model_prompts=model_prompts,
    ),
    name="samaya-ai/promptriever-llama2-7b-v1",
    model_type=["dense"],
    languages=["eng-Latn"],
    open_weights=True,
    revision="01c7f73d771dfac7d292323805ebc428287df4f9-30b14e3813c0fa45facfd01a594580c3fe5ecf23",  # base-peft revision
    release_date="2024-09-15",
    n_parameters=7_000_000_000,
    n_embedding_parameters=None,
    memory_usage_mb=26703,
    max_tokens=4096,
    embed_dim=4096,
    license="apache-2.0",
    training_datasets=set(
        # "samaya-ai/msmarco-w-instructions"
    ),
    reference="https://huggingface.co/samaya-ai/promptriever-llama2-7b-v1",
    similarity_fn_name=ScoringFunction.COSINE,
    framework=["PyTorch", "Tevatron", "safetensors"],
    use_instructions=True,
    citation=PROMPTRIEVER_CITATION,
    public_training_code=None,
    public_training_data=None,
)

promptriever_llama3 = ModelMeta(
    loader=_loader(
        PromptrieverModel,
        base_model_name_or_path="meta-llama/Meta-Llama-3.1-8B",
        peft_model_name_or_path="samaya-ai/promptriever-llama3.1-8b-v1",
        device_map="auto",
        torch_dtype=torch.bfloat16,
        model_prompts=model_prompts,
    ),
    name="samaya-ai/promptriever-llama3.1-8b-v1",
    model_type=["dense"],
    languages=["eng-Latn"],
    open_weights=True,
    revision="48d6d0fc4e02fb1269b36940650a1b7233035cbb-2ead22cfb1b0e0c519c371c63c2ab90ffc511b8a",  # base-peft revision
    training_datasets={
        # "samaya-ai/msmarco-w-instructions",
        "mMARCO-NL",  # translation not trained on
    },
    release_date="2024-09-15",
    n_parameters=8_000_000_000,
    n_embedding_parameters=None,
    memory_usage_mb=30518,
    max_tokens=8192,
    embed_dim=4096,
    license="apache-2.0",
    reference="https://huggingface.co/samaya-ai/promptriever-llama3.1-8b-v1",
    similarity_fn_name=ScoringFunction.COSINE,
    framework=["PyTorch", "Tevatron", "safetensors"],
    use_instructions=True,
    citation=PROMPTRIEVER_CITATION,
    public_training_code=None,
    public_training_data=None,
)

promptriever_llama3_instruct = ModelMeta(
    loader=_loader(
        PromptrieverModel,
        base_model_name_or_path="meta-llama/Meta-Llama-3.1-8B-Instruct",
        peft_model_name_or_path="samaya-ai/promptriever-llama3.1-8b-instruct-v1",
        device_map="auto",
        torch_dtype=torch.bfloat16,
        model_prompts=model_prompts,
    ),
    name="samaya-ai/promptriever-llama3.1-8b-instruct-v1",
    model_type=["dense"],
    languages=["eng-Latn"],
    open_weights=True,
    revision="5206a32e0bd3067aef1ce90f5528ade7d866253f-8b677258615625122c2eb7329292b8c402612c21",  # base-peft revision
    release_date="2024-09-15",
    n_parameters=8_000_000_000,
    n_embedding_parameters=None,
    memory_usage_mb=30518,
    max_tokens=8192,
    embed_dim=4096,
    training_datasets={
        # "samaya-ai/msmarco-w-instructions",
        "mMARCO-NL",  # translation not trained on
    },
    license="apache-2.0",
    reference="https://huggingface.co/samaya-ai/promptriever-llama3.1-8b-instruct-v1",
    similarity_fn_name=ScoringFunction.COSINE,
    framework=["PyTorch", "Tevatron", "safetensors"],
    use_instructions=True,
    citation=PROMPTRIEVER_CITATION,
    public_training_code=None,
    public_training_data=None,
)

promptriever_mistral_v1 = ModelMeta(
    loader=_loader(
        PromptrieverModel,
        base_model_name_or_path="mistralai/Mistral-7B-v0.1",
        peft_model_name_or_path="samaya-ai/promptriever-mistral-v0.1-7b-v1",
        device_map="auto",
        torch_dtype=torch.bfloat16,
        model_prompts=model_prompts,
    ),
    name="samaya-ai/promptriever-mistral-v0.1-7b-v1",
    model_type=["dense"],
    languages=["eng-Latn"],
    open_weights=True,
    revision="7231864981174d9bee8c7687c24c8344414eae6b-876d63e49b6115ecb6839893a56298fadee7e8f5",  # base-peft revision
    release_date="2024-09-15",
    n_parameters=7_000_000_000,
    n_embedding_parameters=131_072_000,
    memory_usage_mb=26703,
    training_datasets={
        # "samaya-ai/msmarco-w-instructions",
        "mMARCO-NL",  # translation not trained on
    },
    max_tokens=4096,
    embed_dim=4096,
    license="apache-2.0",
    reference="https://huggingface.co/samaya-ai/promptriever-mistral-v0.1-7b-v1",
    similarity_fn_name=ScoringFunction.COSINE,
    framework=["PyTorch", "Tevatron", "safetensors"],
    use_instructions=True,
    citation=PROMPTRIEVER_CITATION,
    public_training_code=None,
    public_training_data=None,
)



from mteb.abstasks.retrieval import AbsTaskRetrieval
from mteb.abstasks.task_metadata import TaskMetadata


class SWEbenchLiteReranking(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="SWEbenchLiteRR",
        description="Software Issue Localization.",
        reference="https://www.swebench.com/",
        dataset={
            "path": "mteb/SWEbenchLiteRR",
            "revision": "b4c41d62898febc41cba927b48709949b7664262",
        },
        type="Reranking",
        category="t2t",
        modalities=["text"],
        eval_splits=["train"],
        eval_langs=["eng-Latn", "python-Code"],
        main_score="recall_at_10",
        date=("2023-10-10", "2023-10-10"),
        domains=["Programming", "Written"],
        task_subtypes=["Code retrieval"],
        license="mit",
        annotations_creators="derived",
        dialect=[],
        sample_creation="found",
        prompt={
            "query": "Given a github issue, identify the code that needs to be changed to fix the issue."
        },
        bibtex_citation=r"""
@misc{jimenez2024swebenchlanguagemodelsresolve,
  archiveprefix = {arXiv},
  author = {Carlos E. Jimenez and John Yang and Alexander Wettig and Shunyu Yao and Kexin Pei and Ofir Press and Karthik Narasimhan},
  eprint = {2310.06770},
  primaryclass = {cs.CL},
  title = {SWE-bench: Can Language Models Resolve Real-World GitHub Issues?},
  url = {https://arxiv.org/abs/2310.06770},
  year = {2024},
}
""",
    )



from mteb.abstasks.retrieval import AbsTaskRetrieval
from mteb.abstasks.task_metadata import TaskMetadata


class HagridRetrieval(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="HagridRetrieval",
        dataset={
            "path": "mteb/HagridRetrieval",
            "revision": "ae4f8bebcb82af2028863b778e1eebf4f5f23628",
        },
        reference="https://github.com/project-miracl/hagrid",
        description=(
            "HAGRID (Human-in-the-loop Attributable Generative Retrieval for Information-seeking Dataset)"
            + "is a dataset for generative information-seeking scenarios. It consists of queries"
            + "along with a set of manually labelled relevant passages"
        ),
        type="Retrieval",
        category="t2t",
        modalities=["text"],
        eval_splits=["dev"],
        eval_langs=["eng-Latn"],
        main_score="ndcg_at_10",
        date=("2019-02-01", "2022-10-18"),
        domains=["Encyclopaedic", "Written"],
        task_subtypes=[],
        license="apache-2.0",
        annotations_creators="expert-annotated",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@article{hagrid,
  author = {Ehsan Kamalloo and Aref Jafari and Xinyu Zhang and Nandan Thakur and Jimmy Lin},
  journal = {arXiv:2307.16883},
  title = {{HAGRID}: A Human-LLM Collaborative Dataset for Generative Information-Seeking with Attribution},
  year = {2023},
}
""",
    )



from mteb.abstasks import AbsTaskClustering
from mteb.abstasks.task_metadata import TaskMetadata


class CREMADClustering(AbsTaskClustering):
    label_column_name: str = "label"
    metadata = TaskMetadata(
        name="CREMA_DClustering",
        description="Emotion clustering task with audio data for 6 emotions: Anger, Disgust, Fear, Happy, Neutral, Sad.",
        reference="https://ieeexplore.ieee.org/document/6849440",
        dataset={
            "path": "mteb/crema-d",
            "revision": "a9d0821955c418752248b8310438854e56a1cf34",
        },
        type="AudioClustering",
        category="a2a",
        eval_splits=["train"],
        eval_langs=["eng-Latn"],
        main_score="v_measure",
        date=("2014-01-01", "2014-12-31"),
        domains=["Speech"],
        task_subtypes=["Emotion Clustering"],
        license="http://opendatacommons.org/licenses/odbl/1.0/",  # Open Database License
        annotations_creators="human-annotated",
        dialect=[],
        modalities=["audio"],
        sample_creation="created",
        bibtex_citation=r"""
@article{Cao2014-ih,
  author = {Cao, Houwei and Cooper, David G and Keutmann, Michael K and Gur,
Ruben C and Nenkova, Ani and Verma, Ragini},
  copyright = {https://ieeexplore.ieee.org/Xplorehelp/downloads/license-information/IEEE.html},
  journal = {IEEE Transactions on Affective Computing},
  keywords = {Emotional corpora; facial expression; multi-modal recognition;
voice expression},
  language = {en},
  month = oct,
  number = {4},
  pages = {377--390},
  publisher = {Institute of Electrical and Electronics Engineers (IEEE)},
  title = {{CREMA-D}: Crowd-sourced emotional multimodal actors dataset},
  volume = {5},
  year = {2014},
}
""",
    )
    max_fraction_of_documents_to_embed = None
    input_column_name: str = "audio"

    def dataset_transform(self):
        self.dataset = self.stratified_subsampling(
            self.dataset, seed=self.seed, splits=["train"], label=self.label_column_name
        )
